<?php

namespace Posify\Woocommerce\Entities;

use Exception;
use Illuminate\Database\Eloquent\Model;
use Posify\Core\Entities\Employee as EmployeeEntity;

class Employee extends Model implements EmployeeEntity
{
    protected $table = 'users';

    protected $primaryKey = 'ID';

    protected $guarded = [
        //
    ];

    public $role = null;

    public $timestamps = false;

    /**
     * --------------------------
     * Relationships
     * --------------------------
     */

    public function stores()
    {
        return $this->belongsToMany(
            Store::class,
            'posify_store_has_employees',
            'employee_id',
            'store_id'
        )->withPivot('enabled');
    }

    public function dataInStore()
    {
        return $this->hasMany(StoreEmployee::class, 'employee_id', 'ID');
    }

    /**
     * --------------------------
     * Getters
     * --------------------------
     */

    public function getId(): ?int
    {
        return $this->ID;
    }

    public function getUsername(): ?string
    {
        return $this->user_login;
    }

    public function getFullName(): ?string
    {
        return $this->display_name;
    }

    public function getEmail(): ?string
    {
        return $this->user_email;
    }

    public function getPassword(): ?string
    {
        return $this->user_pass;
    }

    public function getLastLoginTime()
    {
        return get_user_meta($this->getId(), 'posify_last_login', true);
    }

    public function getAvatarUrl()
    {
        return get_avatar_url($this->getId());
    }

    public function getPermissions()
    {
        return array_keys(get_userdata($this->getId())->allcaps);
    }

    public function getRole($force = false)
    {
        if (!$this->role || $force) {
            $metadataRoles = get_userdata($this->getId())->roles;
            $role = reset($metadataRoles); // get first role
            if ($role) {
                $this->setRole($role);
            }
        }

        return $this->role;
    }

    /**
     * --------------------------
     * Setters
     * --------------------------
     */

    public function setId(int $id): void
    {
        $this->ID = $id;
    }

    public function setUsername(?string $username): void
    {
        $this->user_login = $username;
    }

    public function setFullName(?string $fullName): void
    {
        $this->display_name = $fullName;
    }

    public function setEmail(?string $email): void
    {
        $this->user_email = $email;
    }

    public function setPassword(string $password): void
    {
        $this->user_pass = $password;
    }

    public function setRole($role)
    {
        $this->role = $role;
    }

    /**
     * --------------------------
     * Actions
     * --------------------------
     */

    public function save(array $options = [])
    {
        if (!$this->exists) {
            $userData = [
                'user_login' => $this->getUsername(),
                'user_email' => $this->getEmail(),
                'user_pass' => $this->getPassword(),
                'display_name' => $this->getFullName(),
                'role' => $this->getRole(),
            ];

            $idOrError = wp_insert_user($userData);

            if (is_wp_error($idOrError)) {
                throw new Exception(esc_html($idOrError->get_error_message()));
            }

            $this->setId($idOrError);
        }
    }

    /**
     * --------------------------
     * Factories
     * --------------------------
     */

    public static function fromWpUser($user)
    {
        $employee = new static;
        $employee->setId($user->ID);
        $employee->setUsername($user->user_login);
        $employee->setFullName($user->display_name);
        $employee->setEmail($user->user_email);
        $employee->setPassword($user->user_pass);
        $role = get_userdata($user->ID)->roles[0];
        $employee->setRole($role);

        return $employee;
    }

    /**
     * --------------------------
     * Transformers
     * --------------------------
     */

    public function toWpUser()
    {
        return get_user_by('id', $this->getId());
    }
}
