<?php

namespace Posify\Woocommerce\Entities;

use Illuminate\Database\Eloquent\Model;
use Posify\Core\Entities\Register as RegisterEntity;
use Posify\Woocommerce\Authorization\Auth;
use Posify\Woocommerce\Entities\Scopes\OnlyPublishedScope;
use Posify\Woocommerce\Utilities\WpTimestamp;

class Register extends Model implements RegisterEntity
{
    use WpTimestamp;

    protected $table = 'posify_registers';

    protected $casts = [
        'payment_gateways' => 'array',
        'catalog_include_products' => 'array',
        'catalog_include_categories' => 'array',
        'numeric_controller_discount_presets' => 'array',
        'fee_and_discount_presets' => 'array',
        'shipping_methods' => 'array',
        'tax_classes' => 'array',
        'tax_status' => 'array',
        'enabled' => 'boolean',
        'audio_enabled' => 'boolean',
        'scan_enabled' => 'boolean',
        'auto_print_receipt_enabled' => 'boolean',
        'guest_checkout_enabled' => 'boolean',
        'create_product_enabled' => 'boolean',
        'shipping_enabled' => 'boolean',
        'closing_report_enabled' => 'boolean',
        'closing_report_note_enabled' => 'boolean',
        'show_stock_quantity' => 'boolean',
    ];

    protected $guarded = [
        //
    ];

    protected static function booted()
    {
        if (
            Auth::check()
            && Auth::can('posify_only_published_data')
            && posify_is_pro_version()
        ) {
            static::addGlobalScope(new OnlyPublishedScope);
        }
    }

    /**
     * --------------------------
     * Relationships
     * --------------------------
     */

    public function store()
    {
        return $this->belongsTo(Store::class, 'store_id');
    }

    public function receipt()
    {
        return $this->belongsTo(Receipt::class, 'receipt_id');
    }

    public function sessions()
    {
        return $this->hasMany(RegisterSession::class, 'register_id');
    }

    public function currentSession()
    {
        return $this->hasOne(RegisterSession::class, 'register_id')->whereNull('close_time');
    }

    public function getStore()
    {
        return $this->store;
    }

    public function getReceipt()
    {
        return $this->receipt;
    }

    public function getSessions()
    {
        return $this->sessions;
    }

    public function getCurrentSession()
    {
        return $this->currentSession;
    }

    /**
     * --------------------------
     * Getters
     * --------------------------
     */

    public function getId()
    {
        return $this->id;
    }

    public function getStoreId()
    {
        return $this->store_id;
    }

    public function getReceiptId()
    {
        return $this->receipt_id;
    }

    public function getName()
    {
        return $this->name;
    }

    public function getEnabled()
    {
        return $this->enabled;
    }

    public function getPaymentGateways()
    {
        return $this->payment_gateways;
    }

    public function getCatalogIncludeProducts()
    {
        return $this->catalog_include_products;
    }

    public function getCatalogIncludeCategories()
    {
        return $this->catalog_include_categories;
    }

    public function getNumericControllerDiscountPresets()
    {
        return $this->numeric_controller_discount_presets;
    }

    public function getFeeAndDiscountPresets()
    {
        return $this->fee_and_discount_presets;
    }

    public function getShippingMethods()
    {
        return $this->shipping_methods;
    }

    public function getTaxClasses()
    {
        return $this->tax_classes;
    }

    public function getTaxStatus()
    {
        return $this->tax_status;
    }

    public function getClosingReportEnabled()
    {
        return $this->closing_report_enabled;
    }

    public function getClosingReportNoteEnabled()
    {
        return $this->closing_report_note_enabled;
    }

    public function getCatalogIncludeType()
    {
        return $this->catalog_include_type;
    }

    public function getCatalogShowInPosType()
    {
        return $this->catalog_show_in_pos_type;
    }

    public function getAudioEnabled()
    {
        return $this->audio_enabled;
    }

    public function getScanEnabled()
    {
        return $this->scan_enabled;
    }

    public function getGuestCheckoutEnabled()
    {
        return $this->guest_checkout_enabled;
    }

    public function getCreateProductEnabled()
    {
        return $this->create_product_enabled;
    }

    public function getShippingEnabled()
    {
        return $this->shipping_enabled;
    }

    public function getDefaultPaymentGateway()
    {
        return $this->default_payment_gateway;
    }

    public function getDefaultShippingMethod()
    {
        return $this->default_shipping_method;
    }

    public function getDefaultTaxClass()
    {
        return $this->default_tax_class;
    }

    public function getDefaultTaxStatus()
    {
        return $this->default_tax_status;
    }

    public function getAutoPrintReceiptEnabled()
    {
        return $this->auto_print_receipt_enabled;
    }

    public function getShowStockQuantity()
    {
        return $this->show_stock_quantity;
    }

    /**
     * --------------------------
     * Setters
     * --------------------------
     */

    public function setId($id)
    {
        $this->id = $id;
    }

    public function setStoreId($storeId)
    {
        $this->store_id = $storeId;
    }

    public function setReceiptId($receiptId)
    {
        $this->receipt_id = $receiptId;
    }

    public function setName($name)
    {
        $this->name = $name;
    }

    public function setEnabled($enabled)
    {
        $this->enabled = $enabled;
    }

    public function setPaymentGateways($paymentGateways)
    {
        $this->payment_gateways = $paymentGateways;
    }

    public function setCatalogIncludeProducts($catalogIncludeProducts)
    {
        $this->catalog_include_products = $catalogIncludeProducts;
    }

    public function setCatalogIncludeCategories($catalogIncludeCategories)
    {
        $this->catalog_include_categories = $catalogIncludeCategories;
    }

    public function setNumericControllerDiscountPresets($numericControllerDiscountPresets)
    {
        $this->numeric_controller_discount_presets = $numericControllerDiscountPresets;
    }

    public function setFeeAndDiscountPresets($feeAndDiscountPresets)
    {
        $this->fee_and_discount_presets = $feeAndDiscountPresets;
    }

    public function setShippingMethods($shippingMethods)
    {
        $this->shipping_methods = $shippingMethods;
    }

    public function setTaxClasses($taxClasses)
    {
        $this->tax_classes = $taxClasses;
    }

    public function setTaxStatus($taxStatus)
    {
        $this->tax_status = $taxStatus;
    }

    public function setClosingReportEnabled($closingReportEnabled)
    {
        $this->closing_report_enabled = $closingReportEnabled;
    }

    public function setClosingReportNoteEnabled($closingReportNoteEnabled)
    {
        $this->closing_report_note_enabled = $closingReportNoteEnabled;
    }

    public function setCatalogIncludeType($catalogIncludeType)
    {
        $this->catalog_include_type = $catalogIncludeType;
    }

    public function setCatalogShowInPosType($catalogShowInPosType)
    {
        $this->catalog_show_in_pos_type = $catalogShowInPosType;
    }

    public function setAudioEnabled($audioEnabled)
    {
        $this->audio_enabled = $audioEnabled;
    }

    public function setScanEnabled($scanEnabled)
    {
        $this->scan_enabled = $scanEnabled;
    }

    public function setGuestCheckoutEnabled($guestCheckoutEnabled)
    {
        $this->guest_checkout_enabled = $guestCheckoutEnabled;
    }

    public function setCreateProductEnabled($createProductEnabled)
    {
        $this->create_product_enabled = $createProductEnabled;
    }

    public function setShippingEnabled($shippingEnabled)
    {
        $this->shipping_enabled = $shippingEnabled;
    }

    public function setDefaultPaymentGateway($defaultPaymentGateway)
    {
        $this->default_payment_gateway = $defaultPaymentGateway;
    }

    public function setDefaultShippingMethod($defaultShippingMethod)
    {
        $this->default_shipping_method = $defaultShippingMethod;
    }

    public function setDefaultTaxClass($defaultTaxClass)
    {
        $this->default_tax_class = $defaultTaxClass;
    }

    public function setDefaultTaxStatus($defaultTaxStatus)
    {
        $this->default_tax_status = $defaultTaxStatus;
    }

    public function setAutoPrintReceiptEnabled($autoPrintReceiptEnabled)
    {
        $this->auto_print_receipt_enabled = $autoPrintReceiptEnabled;
    }

    public function setShowStockQuantity($showStockQuantity)
    {
        $this->show_stock_quantity = $showStockQuantity;
    }

    /**
     * --------------------------
     * Asserts
     * --------------------------
     */

    public function hasOpenSession(): bool
    {
        return !empty($this->getCurrentSession());
    }

    public function isOpenSessionBelongsToEmployee(Employee $employee): bool
    {
        return $this->hasOpenSession() && $this->getCurrentSession()->getEmployeeId() === $employee->getId();
    }
}
