<?php

namespace Posify\Woocommerce\Entities;

use Illuminate\Database\Eloquent\Model;
use Posify\Core\Entities\Store as StoreEntity;
use Posify\Woocommerce\Authorization\Auth;
use Posify\Woocommerce\Entities\Scopes\OnlyPublishedScope;
use Posify\Woocommerce\Utilities\WpTimestamp;

class Store extends Model implements StoreEntity
{
    use WpTimestamp;

    protected $table = 'posify_stores';

    protected $guarded = [
        //
    ];

    protected $casts = [
        'enabled' => 'boolean',
    ];

    protected static function booted()
    {
        if (
            Auth::check()
            && Auth::can('posify_only_published_data')
            && posify_is_pro_version()
        ) {
            static::addGlobalScope(new OnlyPublishedScope);
        }
    }

    /**
     * --------------------------
     * Relationships
     * --------------------------
     */

    public function dataInStore()
    {
        return $this->hasMany(StoreEmployee::class, 'store_id');
    }

    public function employees()
    {
        $query = $this->belongsToMany(
            Employee::class,
            'posify_store_has_employees',
            'store_id',
            'employee_id'
        )->withPivot('enabled');

        /**
         * If the user is logged in and has the posify_only_published_data capability,
         * then only return the employees that are enabled.
         */
        if (Auth::check() && Auth::can('posify_only_published_data')) {
            $query->wherePivot('enabled', 1);
        }

        return $query;
    }

    public function registers()
    {
        return $this->hasMany(Register::class, 'store_id');
    }

    public function getRegisters()
    {
        return $this->registers;
    }

    public function getEmployees()
    {
        return $this->employees;
    }

    public function stocks()
    {
        return $this->hasMany(Stock::class, 'store_id');
    }

    /**
     * --------------------------
     * Getters
     * --------------------------
     */

    public function getId()
    {
        return $this->id;
    }

    public function getName()
    {
        return $this->name;
    }

    public function getVatNumber()
    {
        return $this->vat_number;
    }

    public function getAddress()
    {
        return $this->address;
    }

    public function getCity()
    {
        return $this->city;
    }

    public function getCountry()
    {
        return $this->country;
    }

    public function getPostcode()
    {
        return $this->postcode;
    }

    public function getPhone()
    {
        return $this->phone;
    }

    public function getEmail()
    {
        return $this->email;
    }

    public function getWebsite()
    {
        return $this->website;
    }

    public function getFax()
    {
        return $this->fax;
    }

    public function getFacebook()
    {
        return $this->facebook;
    }

    public function getTwitter()
    {
        return $this->twitter;
    }

    public function getYoutube()
    {
        return $this->youtube;
    }

    public function getInstagram()
    {
        return $this->instagram;
    }

    public function getEnabled()
    {
        return $this->enabled;
    }

    public function getCountryState()
    {
        return $this->country_state;
    }

    public function getFormattedAddress()
    {
        return sprintf(
            '%s, %s, %s, %s, %s',
            $this->address,
            $this->city,
            $this->country_state,
            $this->postcode,
            $this->country
        );
    }

    /**
     * --------------------------
     * Setters
     * --------------------------
     */

    public function setId($id)
    {
        $this->id = $id;
    }

    public function setName($name)
    {
        return $this->name = $name;
    }

    public function setVatNumber($vatNumber)
    {
        return $this->vat_number = $vatNumber;
    }

    public function setAddress($address)
    {
        return $this->address = $address;
    }

    public function setCity($city)
    {
        return $this->city = $city;
    }

    public function setCountryState($countryState)
    {
        return $this->country_state = $countryState;
    }

    public function setPostcode($postcode)
    {
        return $this->postcode = $postcode;
    }

    public function setPhone($phone)
    {
        return $this->phone = $phone;
    }

    public function setEmail($email)
    {
        return $this->email = $email;
    }

    public function setWebsite($website)
    {
        return $this->website = $website;
    }

    public function setFax($fax)
    {
        return $this->fax = $fax;
    }

    public function setFacebook($facebook)
    {
        return $this->facebook = $facebook;
    }

    public function setTwitter($twitter)
    {
        return $this->twitter = $twitter;
    }

    public function setYoutube($youtube)
    {
        return $this->youtube = $youtube;
    }

    public function setInstagram($instagram)
    {
        return $this->instagram = $instagram;
    }

    public function setEnabled($enabled)
    {
        return $this->enabled = $enabled;
    }

    /**
     * --------------------------
     * Assertions
     * --------------------------
     */

    public function hasEmployee($employee, $role = null, $enabled = true)
    {
        $employeeId = $employee instanceof Employee ? $employee->getId() : $employee;

        $employee = $this->employees()
            ->where('employee_id', $employeeId)
            ->when($enabled, function ($query) {
                $query->where('enabled', true);
            })
            ->first();

        if (!$employee) {
            return false;
        }

        if ($role && $employee->getRole() !== $role) {
            return false;
        }

        return true;
    }
}
