<?php

namespace Posify\Woocommerce\Http\Api\Controllers;

use Posify\Core\DataTransferObjects\RegisterQueryArgs;
use Posify\Core\DataTransferObjects\StoreQueryArgs;
use Posify\Core\Loader;
use Posify\Core\Repositories\RegisterRepository;
use Posify\Core\Repositories\StoreRepository;
use Posify\Woocommerce\Authorization\Auth;
use Posify\Woocommerce\Entities\Employee;
use Posify\Woocommerce\Enums\EmployeeRoles;
use Posify\Woocommerce\Http\Common\Controllers\BaseController;
use WP_REST_Request;

class AuthController extends BaseController
{
    public function login(WP_REST_Request $request)
    {
        $body = $request->get_params();

        $info = [
            'user_login' => $body['username'] ?? null,
            'user_password' => $body['password'] ?? null,
            'remember' => true,
        ];

        $wpUser = wp_signon($info, false);

        if (is_wp_error($wpUser)) {
            return $this->invalidParams(esc_html__('Wrong username or password.', 'posify-for-woocommerce'));
        }

        $user = Employee::fromWpUser($wpUser);
        $role = $user->getRole();

        $allowedRoles = [
            'administrator',
            'shop_manager',
            EmployeeRoles::MANAGER,
            EmployeeRoles::CASHIER,
        ];

        /**
         * Only allowd the roles that we defined can login from this API
         */
        if (!in_array($role, $allowedRoles)) {
            return $this->forbidden();
        }

        wp_clear_auth_cookie();
        wp_set_current_user($user->getId());
        wp_set_auth_cookie($user->getId());

        $store = null;
        $register = null;
        $hasOnlyOneStoreAndRegister = false;
        $storesCount = 0;
        $registerCount = 0;

        $storeRepo = Loader::load(StoreRepository::class);

        if (
            Auth::can('posify_only_their_data')

            /**
             * Only pro version can use this feature
             */
            && posify_is_pro_version()
        ) {
            $args = StoreQueryArgs::make([
                'employee_id' => $user->getId(),
                'enabled' => true
            ]);
            $storesCount = $storeRepo->count($args);

            if ($storesCount == 1) {
                $store = $storeRepo->findWhere($args);
            }
        } else {
            $args = StoreQueryArgs::make([
                'enabled' => true
            ]);
            $storesCount = $storeRepo->count($args);

            if ($storesCount == 1) {
                $store = $storeRepo->findWhere($args);
            }
        }

        $registerRepo = Loader::load(RegisterRepository::class);

        if ($store) {
            $args = RegisterQueryArgs::make([
                'store_id' => $store->getId(),
                'enabled' => true
            ]);

            $registerCount = $registerRepo->count($args);

            if ($registerCount == 1) {
                $register = $registerRepo->findWhere($args);
            }
        }

        $hasOnlyOneStoreAndRegister = $storesCount == 1 && $registerCount == 1;

        return $this->json([
            // TODO: Remove on production
            'token' => base64_encode($user->getUsername() . ':' . $body['password']),

            'user' => [
                'id' => $user->getId(),
                'full_name' => $user->getFullName(),
                'username' => $user->getUsername(),
                'avatar_url' => $user->getAvatarUrl(),
                'last_login_time' => $user->getLastLoginTime(),
                'permissions' => $user->getPermissions(),
                'has_only_one_store_and_register' => $hasOnlyOneStoreAndRegister,
                'store_id' => $store ? $store->getId() : null,
                'register_id' => $register ? $register->getId() : null,
                'role' => $user->getRole(),
            ]
        ], 200);
    }

    public function me()
    {
        $user = Auth::user();

        return $this->json([
            'id' => $user->getId(),
            'full_name' => $user->getFullName(),
            'username' => $user->getUsername(),
            'avatar_url' => $user->getAvatarUrl(),
            'last_login_time' => $user->getLastLoginTime(),
            'permissions' => $user->getPermissions(),
            'role' => $user->getRole(),
        ]);
    }

    public function logout()
    {
        Auth::logout();

        return $this->json([]);
    }
}
