<?php

namespace Posify\Woocommerce\Http\Api\Controllers;

use Posify\Core\DataTransferObjects\ReceiptQueryArgs;
use Posify\Core\Loader;
use Posify\Core\Repositories\ReceiptRepository;
use Posify\Woocommerce\Authorization\Gate;
use Posify\Woocommerce\Http\Common\Controllers\BaseController;
use WP_REST_Request;

class ReceiptController extends BaseController
{
    protected $receiptRepo;

    public function __construct()
    {
        $this->receiptRepo = Loader::load(ReceiptRepository::class);
    }

    public function index(WP_REST_Request $wpRequest)
    {
        if (Gate::deny('receipt', 'read')) {
            return $this->forbidden();
        }

        $args = ReceiptQueryArgs::make($wpRequest->get_params());

        $args->setPage(
            $wpRequest->get_param('page') ?? 1
        );

        $args->setPerPage(
            $wpRequest->get_param('per_page') ?? 15
        );

        $pagination = $this->receiptRepo
            ->paginate($args)
            ->through(function ($receipt) use ($wpRequest) {
                $context = $wpRequest->get_param('context');

                if ($context === 'dropdown') {
                    return $receipt->only(['id', 'name']);
                }

                return $receipt->toArray();
            })
            ->toArray();

        $response = $this->json($pagination['data']);
        $response->header('X-WP-Total', $pagination['total']);
        $response->header('X-WP-TotalPages', $pagination['last_page']);

        return $response;
    }

    public function show(WP_REST_Request $wpRequest)
    {
        $receiptId = $wpRequest->get_param('id');
        $receipt = $this->receiptRepo->find($receiptId);

        if (!$receipt) {
            return $this->notFound(esc_html__('Receipt not found', 'posify-for-woocommerce'));
        }

        if (Gate::deny('receipt', 'read', $receipt)) {
            return $this->forbidden();
        }

        return $this->json(
            $receipt->toArray()
        );
    }

    public function create(WP_REST_Request $wpRequest)
    {
        if (Gate::deny('receipt', 'create')) {
            return $this->forbidden();
        }

        $receipt = posify_create_receipt($wpRequest->get_params());

        return $this->json([
            'id' => $receipt->getId(),
        ], 200);
    }

    public function update(WP_REST_Request $wpRequest)
    {
        $receipt = $this->receiptRepo->find($wpRequest->get_param('id'));

        if (!$receipt) {
            return $this->notFound(esc_html__('Receipt not found', 'posify-for-woocommerce'));
        }

        if (Gate::deny('receipt', 'update', $receipt)) {
            return $this->forbidden();
        }

        $receipt = posify_update_receipt($receipt, $wpRequest->get_params());

        return $this->json([
            'id' => $receipt->id,
        ], 200);
    }

    public function delete(WP_REST_Request $wpRequest)
    {
        $receipt = $this->receiptRepo->find($wpRequest->get_param('id'));

        if (!$receipt) {
            return $this->notFound(esc_html__('Receipt not found', 'posify-for-woocommerce'));
        }

        if (Gate::deny('receipt', 'delete', $receipt)) {
            return $this->forbidden();
        }

        $result = posify_delete_receipt($receipt);

        if (is_wp_error($result)) {
            return $this->invalidParams($result->get_error_message(), $result->get_error_code());
        }

        return $this->json([], 200);
    }

    public function batch(WP_REST_Request $wpRequest)
    {
        $updates = $wpRequest->get_param('update');
        $creates = $wpRequest->get_param('create');
        $deletes = $wpRequest->get_param('delete');

        $results = [];

        if ($updates) {
            foreach ($updates as $update) {
                $receipt = $this->receiptRepo->find($update['id']);
                if (!$receipt) {
                    continue;
                }

                if (Gate::deny('receipt', 'update', $receipt)) {
                    continue;
                }

                posify_update_receipt($receipt, $update);
            }
        }

        if ($creates) {
            foreach ($creates as $create) {
                if (Gate::deny('receipt', 'create')) {
                    continue;
                }

                $receipt = posify_create_receipt($create);
                $results['create'][] = [
                    'id' => $receipt->id,
                ];
            }
        }

        if ($deletes) {
            foreach ($deletes as $delete) {
                $receipt = $this->receiptRepo->find($delete['id']);
                if (!$receipt) {
                    continue;
                }

                if (Gate::deny('receipt', 'delete', $receipt)) {
                    continue;
                }

                posify_delete_receipt($receipt);
            }
        }

        return $this->json($results, 200);
    }
}
