<?php

namespace Posify\Woocommerce\Http\Api\Controllers;

use Posify\Core\DataTransferObjects\RegisterCashQueryArgs;
use Posify\Core\Loader;
use Posify\Core\Repositories\RegisterCashRepository;
use Posify\Core\Repositories\RegisterRepository;
use Posify\Woocommerce\Authorization\Auth;
use Posify\Woocommerce\Authorization\Gate;
use Posify\Woocommerce\Http\Common\Controllers\BaseController;
use WP_REST_Request;

class RegisterCashController extends BaseController
{
    protected $registerCashRepo;
    protected $registerRepo;

    public function __construct()
    {
        $this->registerRepo = Loader::load(RegisterRepository::class);
        $this->registerCashRepo = Loader::load(RegisterCashRepository::class);
    }

    public function index(WP_REST_Request $wpRequest)
    {
        if (Gate::deny('register_cash', 'read')) {
            return $this->forbidden();
        }

        $args = RegisterCashQueryArgs::make($wpRequest->get_params());

        if (Auth::can('posify_only_their_data')) {
            $args->employee_id = Auth::user()->getId();
        }

        $args->setPage(
            $wpRequest->get_param('page') ?? 1
        );

        $args->setPerPage(
            $wpRequest->get_param('per_page') ?? 15
        );

        $pagination = $this->registerCashRepo->paginate($args)->toArray();

        $response = $this->json($pagination['data']);
        $response->header('X-WP-Total', $pagination['total']);
        $response->header('X-WP-TotalPages', $pagination['last_page']);

        return $response;
    }

    public function getListByCurrentRegisterSession(WP_REST_Request $wpRequest)
    {
        $register = $this->registerRepo->find($wpRequest->get_param('register_id'));

        if (!$register) {
            return $this->notFound(esc_html__('Register not found', 'posify-for-woocommerce'));
        }

        $currentSession = $register->getCurrentSession();

        if (!$currentSession) {
            return $this->notFound(esc_html__('Session not found', 'posify-for-woocommerce'));
        }

        if (
            Gate::deny('register_cash', 'read')
            || Gate::deny('register', 'read', $register)
            || Gate::deny('register_session', 'read', $currentSession)
        ) {
            return $this->forbidden();
        }

        $args = RegisterCashQueryArgs::make($wpRequest->get_params());
        $args->session_id = $currentSession->getId();

        $args->setPage(
            $wpRequest->get_param('page') ?? 1
        );

        $args->setWith(['employee' => function ($query) {
            $query->select(['ID', 'display_name', 'user_login']);
        }]);

        $data = $this->registerCashRepo
            ->get($args)
            ->map(function ($cash) {
                return [
                    'id' => $cash->getId(),
                    'amount' => $cash->getAmount(),
                    'description' => $cash->getDescription(),
                    'employee' => [
                        'id' => $cash->getEmployee()->getId(),
                        'username' => $cash->getEmployee()->getUsername(),
                        'full_name' => $cash->getEmployee()->getFullName(),
                        'avatar_url' => $cash->getEmployee()->getAvatarUrl(),
                    ],
                    'created_at' => $cash->getCreatedAt(),
                    'updated_at' => $cash->getUpdatedAt(),
                ];
            })->toArray();

        $response = $this->json($data);

        return $response;
    }

    public function create(WP_REST_Request $wpRequest)
    {
        $register = $this->registerRepo->find($wpRequest->get_param('register_id'));

        if (!$register) {
            return $this->notFound(esc_html__('Register not found', 'posify-for-woocommerce'));
        }

        $currentSession = $register->getCurrentSession();

        if (!$currentSession) {
            return $this->notFound(esc_html__('Register session not found', 'posify-for-woocommerce'));
        }

        if (Gate::deny('register_cash', 'create', $currentSession)) {
            return $this->forbidden();
        }

        $cash = posify_add_cash_to_register_session($currentSession, $wpRequest->get_params());

        return $this->json([
            'id' => $cash->getId(),
        ], 200);
    }
}
