<?php

namespace Posify\Woocommerce\Http\Api\Controllers;

use Posify\Core\DataTransferObjects\RegisterQueryArgs;
use Posify\Core\Loader;
use Posify\Core\Repositories\RegisterRepository;
use Posify\Core\Repositories\StoreRepository;
use Posify\Woocommerce\Authorization\Auth;
use Posify\Woocommerce\Authorization\Gate;
use Posify\Woocommerce\Http\Common\Controllers\BaseController;
use WP_REST_Request;

class RegisterController extends BaseController
{
    protected $registerRepo;
    protected $storeRepo;

    public function __construct()
    {
        $this->registerRepo = Loader::load(RegisterRepository::class);
        $this->storeRepo = Loader::load(StoreRepository::class);
    }

    public function index(WP_REST_Request $wpRequest)
    {
        $args = RegisterQueryArgs::make($wpRequest->get_params());

        $lazyLoad = [
            'store' => function ($query) {
                $query->select(['id', 'name']);
            },
            'receipt' => function ($query) {
                $query->select(['id', 'name']);
            },
        ];

        $args->setWith($lazyLoad);

        /**
         * Just show the registers that the user has permission to read
         */
        if (Auth::can('posify_only_their_data')) {
            $args->store_has_employee_id = Auth::id();
        }

        $args->setPage(
            $wpRequest->get_param('page') ?? 1
        );

        $args->setPerPage(
            $wpRequest->get_param('per_page') ?? 15
        );

        $pagination = $this->registerRepo
            ->paginate($args)
            ->through(function ($register) use ($wpRequest) {
                $context = $wpRequest->get_param('context');

                if ($context === 'dropdown') {
                    return $register->only(['id', 'name']);
                }

                return $register->toArray();
            })
            ->toArray();

        $response = $this->json($pagination['data']);
        $response->header('X-WP-Total', $pagination['total']);
        $response->header('X-WP-TotalPages', $pagination['last_page']);

        return $response;
    }

    public function show(WP_REST_Request $wpRequest)
    {
        $registerId = $wpRequest->get_param('id');
        $register = $this->registerRepo->find($registerId);

        if (!$register) {
            return $this->notFound(esc_html__('Register not found', 'posify-for-woocommerce'));
        }

        if (Gate::deny('register', 'read', $register)) {
            return $this->forbidden();
        }

        $lazyLoad = [
            'store' => function ($query) {
                $query->select(['id', 'name']);
            },
            'receipt' => function ($query) {
                $query->select(['id', 'name']);
            },
        ];
        $response = $register->load($lazyLoad)->toArray();

        return $this->json($response);
    }

    public function create(WP_REST_Request $wpRequest)
    {
        $storeId = $wpRequest->get_param('store_id');

        if (!$storeId) {
            return $this->invalidParams(
               esc_html__('Store is required', 'posify-for-woocommerce'),
            );
        }

        $store = $this->storeRepo->find($storeId);

        if (!$store) {
            return $this->notFound(
               esc_html__('Store not found', 'posify-for-woocommerce'),
            );
        }

        if (Gate::deny('register', 'create', $store)) {
            return $this->forbidden();
        }

        $register = posify_create_register($wpRequest->get_params());

        return $this->json([
            'id' => $register->getId(),
        ], 200);
    }

    public function update(WP_REST_Request $wpRequest)
    {
        $register = $this->registerRepo->find($wpRequest->get_param('id'));

        if (!$register) {
            return $this->notFound(esc_html__('Register not found', 'posify-for-woocommerce'));
        }

        if (Gate::deny('register', 'update', $register)) {
            return $this->forbidden();
        }

        posify_update_register($register, $wpRequest->get_params());

        return $this->json([
            'id' => $register->getId(),
        ], 200);
    }

    public function delete(WP_REST_Request $wpRequest)
    {
        $register = $this->registerRepo->find($wpRequest->get_param('id'));

        if (!$register) {
            return $this->notFound(esc_html__('Register not found', 'posify-for-woocommerce'));
        }

        if (Gate::deny('register', 'delete', $register)) {
            return $this->forbidden();
        }

        posify_delete_register($register);

        return $this->json([], 200);
    }

    public function batch(WP_REST_Request $wpRequest)
    {
        $updates = $wpRequest->get_param('update');
        $creates = $wpRequest->get_param('create');
        $deletes = $wpRequest->get_param('delete');

        $results = [];

        if ($updates) {
            foreach ($updates as $update) {
                $register = $this->registerRepo->find($update['id']);

                if (Gate::deny('register', 'update', $register)) {
                    continue;
                }

                posify_update_register($register, $update);
            }
        }

        if ($creates) {
            foreach ($creates as $create) {
                $storeId = $create['store_id'] ?? null;

                if (!$storeId) {
                    continue;
                }

                $store = $this->storeRepo->find($storeId);

                if (!$store) {
                    continue;
                }

                if (Gate::deny('store', 'update', $store)) {
                    continue;
                }

                $register = posify_create_register($create);

                $results['create'][] = [
                    'id' => $register->getId(),
                ];
            }
        }

        if ($deletes) {
            foreach ($deletes as $delete) {
                $register = $this->registerRepo->find($delete['id']);

                if (Gate::deny('register', 'update', $register)) {
                    continue;
                }

                posify_delete_register($register);
            }
        }

        return $this->json($results, 200);
    }
}
