<?php

namespace Posify\Woocommerce\Http\Api\Controllers;

use Posify\Core\DataTransferObjects\RegisterSessionQueryArgs;
use Posify\Core\Entities\RegisterSession;
use Posify\Core\Loader;
use Posify\Core\Repositories\EmployeeRepository;
use Posify\Core\Repositories\RegisterRepository;
use Posify\Core\Repositories\RegisterSessionRepository;
use Posify\Core\UseCases\CloseRegisterSession;
use Posify\Core\UseCases\OpenRegisterSession;
use Posify\Core\UseCases\TakeOverRegisterSession;
use Posify\Woocommerce\Authorization\Auth;
use Posify\Woocommerce\Authorization\Gate;
use Posify\Woocommerce\Http\Api\Response\DetailRegisterSessionResponse;
use Posify\Woocommerce\Http\Common\Controllers\BaseController;
use Posify\Woocommerce\Reports\RegisterSessionReport;
use WP_REST_Request;

class RegisterSessionController extends BaseController
{
    protected $registerSessionRepo;
    protected $employeeRepo;
    protected $registerRepo;

    public function __construct()
    {
        $this->registerSessionRepo = Loader::load(RegisterSessionRepository::class);
        $this->employeeRepo = Loader::load(EmployeeRepository::class);
        $this->registerRepo = Loader::load(RegisterRepository::class);
    }

    /**
     * Show all sessions
     */
    public function index(WP_REST_Request $wpRequest)
    {
        if (Gate::deny('register_session', 'read')) {
            return $this->forbidden();
        }

        $args = RegisterSessionQueryArgs::make($wpRequest->get_params());

        /**
         * If user can 'posify_manage_store' and 'posify_only_their_data'
         * That means they can only manage their own store
         *
         * So we just show the sessions that belong to their store
         */
        if (
            Auth::can('posify_manage_store')
            && Auth::can('posify_only_their_data')
        ) {
            $args->store_has_employee_id = Auth::user()->getId();
        }

        /**
         * But if user can only 'posify_only_their_data'
         * That means they can only manage their own sessions
         *
         * So we just show the sessions that belong to them
         */
        else if (Auth::can('posify_only_their_data')) {
            $args->employee_id = Auth::user()->getId();
        }

        $lazyLoad = [
            'store' => function ($query) {
                $query->select(['id', 'name']);
            },
            'register' => function ($query) {
                $query->select(['id', 'name']);
            },
        ];
        $args->setWith($lazyLoad);

        $args->setPage(
            $wpRequest->get_param('page') ?? 1
        );

        $args->setPerPage(
            $wpRequest->get_param('per_page') ?? 15
        );

        $pagination = $this->registerSessionRepo
            ->paginate($args)
            ->through(function ($session) {
                return array_merge([
                    'total_sales' => $session->getTotalSales(),
                ], $session->toArray());
            })
            ->toArray();

        $response = $this->json($pagination['data']);
        $response->header('X-WP-Total', $pagination['total']);
        $response->header('X-WP-TotalPages', $pagination['last_page']);

        return $response;
    }

    /**
     * Show detail a session by id
     */
    public function show(WP_REST_Request $wpRequest)
    {
        $session = $this->registerSessionRepo->find($wpRequest->get_param('id'));

        if (!$session) {
            return $this->notFound(esc_html__('Session not found', 'posify-for-woocommerce'));
        }

        if (Gate::deny('register_session', 'read', $session)) {
            return $this->forbidden();
        }

        $context = $wpRequest->get_param('context');

        return $this->responseSession($session, $context);
    }

    public function showCurrent(WP_REST_Request $wpRequest)
    {
        $register = $this->registerRepo->find($wpRequest->get_param('register_id'));

        if (!$register) {
            return $this->notFound(esc_html__('Register not found', 'posify-for-woocommerce'));
        }

        $currentSession = $register->getCurrentSession();

        if (!$currentSession) {
            return $this->json(null);
        }

        if (Gate::deny('register_session', 'read', $currentSession)) {
            return $this->forbidden();
        }

        $context = $wpRequest->get_param('context');

        return $this->responseSession($currentSession, $context);
    }

    /**
     * Open a session for a register
     */
    public function open(WP_REST_Request $wpRequest)
    {
        $register = $this->registerRepo->find($wpRequest->get_param('register_id'));

        if (!$register) {
            return $this->notFound(esc_html__('Register not found', 'posify-for-woocommerce'));
        }

        if (Gate::deny('register_session', 'create', $register)) {
            return $this->forbidden();
        }

        $employee = Auth::user();

        if ($register->hasOpenSession()) {
            if ($register->isOpenSessionBelongsToEmployee($employee) == false) {
                return $this->jsonError(
                    'pos_current_session_taken',
                   esc_html__('There is a session already taken for this register', 'posify-for-woocommerce'),
                    ['status' => 400]
                );
            }

            $session = $register->getCurrentSession();
        } else {
            $time = posify_now()->format('Y-m-d H:i:s');
            $session = OpenRegisterSession::make($register, $employee, $time)->handle();
        }

        return $this->json([
            'id' => $session->id,
        ], 200);
    }

    /**
     * Close a current session for a register
     */
    public function closeCurrent(WP_REST_Request $wpRequest)
    {
        $register = $this->registerRepo->find($wpRequest->get_param('register_id'));

        if (!$register) {
            return $this->notFound(esc_html__('Register not found', 'posify-for-woocommerce'));
        }

        $currentSession = $register->getCurrentSession();

        if (!$currentSession) {
            return $this->notFound(esc_html__('Session not found', 'posify-for-woocommerce'));
        }

        if (Gate::deny('register_session', 'delete', $currentSession)) {
            return $this->forbidden();
        }

        $note = $wpRequest->get_param('note');

        $time = posify_now()->format('Y-m-d H:i:s');
        CloseRegisterSession::make($currentSession, $time, $note)->handle();

        return $this->json([
            //
        ], 200);
    }

    /**
     * Close a session for a register
     */
    public function close(WP_REST_Request $wpRequest)
    {
        $session = $this->registerSessionRepo->find($wpRequest->get_param('id'));

        if (!$session) {
            return $this->notFound(esc_html__('Session not found', 'posify-for-woocommerce'));
        }

        if (Gate::deny('register_session', 'delete', $session)) {
            return $this->forbidden();
        }

        $note = $wpRequest->get_param('note');

        $time = posify_now()->format('Y-m-d H:i:s');
        CloseRegisterSession::make($session, $time, $note)->handle();

        return $this->json([
            'close_time' => $session->getCloseTime(),
        ], 200);
    }

    /**
     * Take over a session for a register
     */
    public function takeOver(WP_REST_Request $wpRequest)
    {
        $register = $this->registerRepo->find($wpRequest->get_param('register_id'));

        if (!$register) {
            return $this->notFound(esc_html__('Register not found', 'posify-for-woocommerce'));
        }

        $currentSession = $register->getCurrentSession();

        if (!$currentSession) {
            return $this->notFound();
        }

        if (Gate::deny('register_session', 'update', $currentSession)) {
            return $this->forbidden();
        }

        $employee = Auth::user();

        TakeOverRegisterSession::make($currentSession, $employee)->handle($currentSession);

        return $this->json([
            'id' => $currentSession->id,
        ], 200);
    }

    private function responseSession(RegisterSession $session, $context = null)
    {
        $session->load([
            'store' => function ($query) {
                $query->select('id', 'name');
            },
            'register' => function ($query) {
                $query->select('id', 'name');
            },
        ]);

        $reportQueryArgs = [
            'session' => $session,
        ];

        if (Auth::can('posify_only_their_data')) {
            $reportQueryArgs['employee'] = Auth::user();
        }

        $report = RegisterSessionReport::make($reportQueryArgs);

        return $this->json(
            DetailRegisterSessionResponse::make($session, $report, $context)->toArray()
        );
    }
}
