<?php

namespace Posify\Woocommerce\Http\Api\Controllers\ReportController;

use Carbon\Carbon;
use DateTime;
use Posify\Core\DataTransferObjects\IntervalReportQuery;
use Posify\Core\DataTransferObjects\OverviewReportQuery;
use Posify\Woocommerce\Authorization\Auth;
use Posify\Woocommerce\Authorization\Gate;
use Posify\Woocommerce\Http\Common\Controllers\BaseController;
use Posify\Woocommerce\Reports\IntervalReport;
use Posify\Woocommerce\Reports\OverviewReport;
use WP_REST_Request;

class OverviewReportAction extends BaseController
{
    protected $baseQueryArgs = [];

    public function __invoke(WP_REST_Request $request)
    {
        if (Gate::deny('report', 'read')) {
            return $this->forbidden();
        }

        $this->baseQueryArgs = [
            // Required params
            'start_time' => $request->get_param('start_time'),
            'end_time' => $request->get_param('end_time'),

            // Optional params
            'interval' => $request->get_param('interval'),
            'store_id' => $request->get_param('store_id'),
            'register_id' => $request->get_param('register_id'),
            'employee_id' => $request->get_param('employee_id'),
        ];

        if (empty($this->baseQueryArgs['start_time']) || empty($this->baseQueryArgs['end_time'])) {
            return $this->invalidParams(esc_html__('start_time and end_time are required', 'posify-for-woocommerce'));
        }

        if (!$this->validateDateFormat($this->baseQueryArgs['start_time']) || !$this->validateDateFormat($this->baseQueryArgs['end_time'])) {
            return $this->invalidParams(esc_html__('start_time and end_time must be in Y-m-d format', 'posify-for-woocommerce'));
        }

        $compareWith = $request->get_param('compare_with');

        return $this->json([
            'current' => $this->getReport(),
            'compare' => $this->getReport(true, $compareWith),
        ], 200);
    }

    /**
     * Get report
     *
     * @param bool $isComareReport Whether to get compare report
     * @param string $compareWith Compare with previous period or previous year
     */
    private function getReport($isComareReport = false, $compareWith = 'previous-period')
    {
        $startTime = Carbon::createFromFormat('Y-m-d', $this->baseQueryArgs['start_time'])->startOfday();
        $endTime = Carbon::createFromFormat('Y-m-d', $this->baseQueryArgs['end_time'])->endOfDay();

        if ($isComareReport) {
            if ($compareWith == 'previous-period') {
                [$startTime, $endTime] = posify_get_previous_date($startTime, $endTime);
            } elseif ($compareWith == 'previous-year') {
                $startTime = $startTime->clone()->subYear()->startOfDay();
                $endTime = $endTime->clone()->subYear()->startOfDay();
            }
        }

        $queryArgs = OverviewReportQuery::make([
            'start_time' => $startTime->format('Y-m-d H:i:s'),
            'end_time' => $endTime->format('Y-m-d H:i:s'),
            'store_id' => $this->baseQueryArgs['store_id'] ?? null,
            'register_id' => $this->baseQueryArgs['register_id'] ?? null,
            'employee_id' => $this->baseQueryArgs['employee_id'] ?? null,
        ]);

        $owner = null;

        if (Auth::can('posify_only_their_data')) {
            $owner = Auth::user();
        }

        $overviewReport = OverviewReport::make($queryArgs, $owner);
        $overviewReport->calculate();

        return [
            'totals' => [
                'total_orders' => $overviewReport->getTotalOrders(),
                'total_sales' => $overviewReport->getTotalSales(),
                'total_items' => $overviewReport->getTotalItems(),
                'total_discounts' => $overviewReport->getTotalDiscounts(),
                'total_net_sales' => $overviewReport->getTotalNetSales(),
                'avg_order_value' => $overviewReport->getAvgOrderValue(),
                'avg_items_per_order' => $overviewReport->getAvgItemsPerOrder(),
                'total_customers' => $overviewReport->getTotalCustomers(),
            ],

            'intervals' => IntervalReport::make(IntervalReportQuery::make([
                'start_time' => $startTime->format('Y-m-d H:i:s'),
                'end_time' => $endTime->format('Y-m-d H:i:s'),
                'interval' => $this->baseQueryArgs['interval'] ?? null,
                'store_id' => $this->baseQueryArgs['store_id'] ?? null,
                'register_id' => $this->baseQueryArgs['register_id'] ?? null,
                'employee_id' => $this->baseQueryArgs['employee_id'] ?? null,
            ]))->toArray(),
        ];
    }

    private function validateDateFormat($date, $format = 'Y-m-d')
    {
        $d = DateTime::createFromFormat($format, $date);

        return $d && $d->format($format) == $date;
    }
}
