<?php

namespace Posify\Woocommerce\Http\Api\Controllers;

use Posify\Core\DataTransferObjects\UpdateSettingData;
use Posify\Core\Loader;
use Posify\Core\Repositories\ReceiptRepository;
use Posify\Core\Repositories\RegisterRepository;
use Posify\Core\Repositories\SettingRepository;
use Posify\Core\Repositories\StoreRepository;
use Posify\Core\UseCases\UpdateSetting;
use Posify\CustomerLicense\LicenseManager;
use Posify\Woocommerce\Authorization\Gate;
use Posify\Woocommerce\Http\Common\Controllers\BaseController;
use WC_Payment_Gateways;
use WC_Shipping;
use WC_Tax;
use WP_REST_Request;

class SettingController extends BaseController
{
    protected $settingRepo;
    protected $storeRepo;
    protected $registerRepo;
    protected $receiptRepo;

    public function __construct()
    {
        $this->settingRepo = Loader::load(SettingRepository::class);
        $this->storeRepo = Loader::load(StoreRepository::class);
        $this->registerRepo = Loader::load(RegisterRepository::class);
        $this->receiptRepo = Loader::load(ReceiptRepository::class);
    }

    public function show(WP_REST_Request $wpRequest)
    {
        $settings = [
            'version' => POSIFY_VERSION,
            'plan' => posify_get_plan(),
            'login_background_url' => $this->settingRepo->get('login_background_url'),
            'login_background_color' => $this->settingRepo->get('login_background_color'),
            'primary_color' => $this->settingRepo->get('primary_color'),
            'secondary_color' => $this->settingRepo->get('secondary_color'),
            'info_color' => $this->settingRepo->get('info_color'),
            'positive_color' => $this->settingRepo->get('positive_color'),
            'negative_color' => $this->settingRepo->get('negative_color'),
            'notify_no_stock_amount' => $this->settingRepo->get('notify_no_stock_amount'),
            'multistock_enabled' => (int) $this->settingRepo->get('multistock_enabled'),
            'multistock_condition' => $this->settingRepo->get('multistock_condition'),
            'vat_field_label' => $this->settingRepo->get('vat_field_label'),
            'product_scan_field' => $this->settingRepo->get('product_scan_field'),
            'logo_url' => $this->settingRepo->get('logo_url'),
            'address_format' => $this->settingRepo->get('address_format'),
            'payment_gateways' => $this->settingRepo->get('payment_gateways'),
            'shipping_methods' => $this->settingRepo->get('shipping_methods'),

            'tax_classes' => $this->settingRepo->get('tax_classes'),
            'tax_status' => $this->settingRepo->get('tax_status'),
            'tax_enabled' => get_option('woocommerce_calc_taxes') == 'yes' ? 1 : 0,
            'prices_include_tax' => get_option('woocommerce_prices_include_tax') == 'yes' ? 1 : 0,
            'tax_based_on' => get_option('woocommerce_tax_based_on'),

            'currency' => [
                'code' => get_woocommerce_currency(),
                'currency_pos' => get_option('woocommerce_currency_pos'),
                'thousand_sep' => get_option('woocommerce_price_thousand_sep'),
                'decimal_sep' => get_option('woocommerce_price_decimal_sep'),
                'num_decimals' => get_option('woocommerce_price_num_decimals'),
                'symbol' => html_entity_decode(get_woocommerce_currency_symbol(), ENT_HTML5),
                'symbol_pos' => get_option('woocommerce_currency_pos'),
            ],

            'shop_address' => [
                'country_state' => get_option('woocommerce_default_country'),
                'city' => get_option('woocommerce_store_city'),
                'postcode' => get_option('woocommerce_store_postcode'),
            ],

            'data' => [
                'languages' => array_merge(['en'], get_available_languages()),
                'shipping_methods' => collect(WC_Shipping::instance()->get_shipping_methods())
                    ->map(function ($shipping) {
                        return [
                            'id' => $shipping->id,
                            'title' => $shipping->method_title
                        ];
                    })
                    ->values()
                    ->toArray(),

                'payment_gateways' => collect(WC_Payment_Gateways::instance()->payment_gateways())
                    ->map(function ($gateway) {
                        return [
                            'id' => $gateway->id,
                            'title' => $gateway->method_title
                        ];
                    })
                    ->values()
                    ->toArray(),

                'tax_status' => [
                    [
                        'id' => 'taxable',
                        'name' => 'Taxable',
                    ],
                    [
                        'id' => 'none',
                        'name' => 'None',
                    ],
                    [
                        'id' => 'shipping',
                        'name' => 'Shipping',
                    ],
                ],
                'tax_classes' => collect(wc_get_product_tax_class_options())
                    ->map(function ($value, $key) {
                        return [
                            'id' => empty($key) ? 'standard' : $key,
                            'title' => $value,
                            'rates' => collect(WC_Tax::get_rates_for_tax_class($key))
                                ->map(function ($rate) {
                                    return [
                                        'id' => $rate->tax_rate_id,
                                        'country' => $rate->tax_rate_country,
                                        'state' => $rate->tax_rate_state,
                                        'postcode' => $rate->postcode ?? '',
                                        'city' => $rate->city ?? '',
                                        'rate' => $rate->tax_rate,
                                        'name' => $rate->tax_rate_name,
                                        'priority' => $rate->tax_rate_priority,
                                        'compound' => $rate->tax_rate_compound,
                                        'shipping' => $rate->tax_rate_shipping,
                                        'order' => $rate->tax_rate_order,
                                        'priority' => $rate->tax_rate_priority,
                                    ];
                                })
                                ->sortBy('order')
                                ->values()
                                ->toArray(),
                        ];
                    })
                    ->values()
                    ->toArray(),
            ],
        ];

        $context = $wpRequest->get_param('context');
        $requestFromPlan = $wpRequest->get_param('plan');

        if (
            posify_is_pro_version() == false
            && $requestFromPlan == 'free'
            && $context == 'admin_settings'
        ) {
            $defaultStore = $this->storeRepo->findOrCreateDefaultStore();
            $defaultReceipt = $this->receiptRepo->findOrCreateDefaultReceipt();
            $defaultRegister = $this->registerRepo->findOrCreateDefaultRegister($defaultStore, $defaultReceipt);

            /**
             * We want to ensure default store and register is enabled
             */
            if ($defaultStore->getEnabled() == false) {
                posify_update_store($defaultStore, [
                    'enabled' => true,
                ]);
            }

            if ($defaultRegister->getEnabled() == false) {
                posify_update_register($defaultRegister, [
                    'enabled' => true,
                ]);
            }

            $settings['default_store'] = $defaultStore ? $defaultStore->toArray() : null;
            $settings['default_register'] = $defaultRegister ? $defaultRegister->toArray() : null;
            $settings['default_receipt'] = $defaultReceipt ? $defaultReceipt->toArray() : null;
        }

        if (posify_is_pro_version()) {
            $settings['license_string'] = null;
            $settings['license'] = null;

            $licenseManager = LicenseManager::getInstance();
            $license = $licenseManager->getLicense();

            if ($license) {
                $settings['license'] = $license->toArray();
            }

            if ($context == 'admin_settings' && $license) {
                $settings['license_string'] = $licenseManager->getLicenseAsString();
            }
        }

        return $this->json($settings);
    }

    public function update(WP_REST_Request $wpRequest)
    {
        if (Gate::deny('setting', 'update')) {
            return $this->forbidden();
        }

        $body = $wpRequest->get_params();

        UpdateSetting::make(
            UpdateSettingData::make($body)
        )->handle();

        return $this->json([], 200);
    }
}
