<?php

namespace Posify\Woocommerce\Http\Api\Controllers;

use Posify\Core\DataTransferObjects\StoreQueryArgs;
use Posify\Core\Loader;
use Posify\Core\Repositories\StoreRepository;
use Posify\Woocommerce\Authorization\Auth;
use Posify\Woocommerce\Authorization\Gate;
use Posify\Woocommerce\Http\Common\Controllers\BaseController;
use WP_REST_Request;

class StoreController extends BaseController
{
    protected $storeRepo;

    public function __construct()
    {
        $this->storeRepo = Loader::load(StoreRepository::class);
    }

    public function index(WP_REST_Request $wpRequest)
    {
        if (Gate::deny('store', 'read')) {
            return $this->forbidden();
        }

        $args = StoreQueryArgs::make($wpRequest->get_params());

        /**
         * Limit the store to the one that the user is assigned to
         * But just for pro version
         */
        if (Auth::can('posify_only_their_data')
            && posify_is_pro_version()
        ) {
            $args->employee_id = Auth::user()->getId();
        }

        $args->setWithCount(['employees', 'registers']);
        $args->setWith(['registers']);

        $args->setPage(
            $wpRequest->get_param('page') ?? 1
        );

        $args->setPerPage(
            $wpRequest->get_param('per_page') ?? 15
        );

        $pagination = $this->storeRepo->paginate($args)
            ->through(function ($store) use ($wpRequest) {
                $context = $wpRequest->get_param('context');

                if ($context === 'dropdown') {
                    return $store->only(['id', 'name']);
                }

                if ($context === 'dropdown_with_registers') {
                    return [
                        'id' => $store->getId(),
                        'name' => $store->getName(),
                        'enabled' => $store->getEnabled(),
                        'registers' => $store->getRegisters()->map(function ($register) {
                            return [
                                'id' => $register->getId(),
                                'name' => $register->getName(),
                                'enabled' => $register->getEnabled(),
                            ];
                        }),
                    ];
                }

                return $store->toArray();
            })
            ->toArray();

        $response = $this->json($pagination['data']);
        $response->header('X-WP-Total', $pagination['total']);
        $response->header('X-WP-TotalPages', $pagination['last_page']);

        return $response;
    }

    public function show(WP_REST_Request $wpRequest)
    {
        $storeId = $wpRequest->get_param('id');
        $store = $this->storeRepo->find($storeId);

        if (!$store) {
            return $this->notFound();
        }

        if (Gate::deny('store', 'read', $store)) {
            return $this->forbidden();
        }

        return $this->json(
            $store->toArray()
        );
    }

    public function create(WP_REST_Request $wpRequest)
    {
        if (Gate::deny('store', 'create')) {
            return $this->forbidden();
        }

        $store = posify_create_store($wpRequest->get_params());

        /**
         * If user has 'posify_manage_store' and 'posify_only_their_data'
         * That means they can only manage their own store
         *
         * So we need to assign the store to them after creating
         * Because if not, they can't see the store they just created
         */
        if (
            Auth::can('posify_manage_store')
            && Auth::can('posify_only_their_data')
        ) {
            posify_add_employee_to_store($store, Auth::user(), true);
        }

        return $this->json([
            'id' => $store->getId(),
        ], 200);
    }

    public function update(WP_REST_Request $wpRequest)
    {
        $store = $this->storeRepo->find($wpRequest->get_param('id'));

        if (!$store) {
            return $this->notFound(esc_html__('Store not found', 'posify-for-woocommerce'));
        }

        if (Gate::deny('store', 'update', $store)) {
            return $this->forbidden();
        }

        $store = posify_update_store($store, $wpRequest->get_params());

        return $this->json([
            'id' => $store->getId(),
        ], 200);
    }

    public function delete(WP_REST_Request $wpRequest)
    {
        $store = $this->storeRepo->find($wpRequest->get_param('id'));

        if (!$store) {
            return $this->notFound();
        }

        if (Gate::deny('store', 'delete', $store)) {
            return $this->forbidden();
        }

        posify_delete_store($store);

        return $this->json([], 200);
    }

    public function batch(WP_REST_Request $wpRequest)
    {
        $updates = $wpRequest->get_param('update');
        $creates = $wpRequest->get_param('create');
        $deletes = $wpRequest->get_param('delete');

        $results = [];

        if ($updates) {
            foreach ($updates as $update) {
                $store = $this->storeRepo->find($update['id']);
                if (!$store) {
                    continue;
                }

                if (Gate::deny('store', 'update', $store)) {
                    continue;
                }

                posify_update_store($store, $update);
            }
        }

        if ($creates) {
            foreach ($creates as $create) {
                if (Gate::deny('store', 'create')) {
                    continue;
                }

                $store = posify_create_store($create);
                $results['create'][] = [
                    'id' => $store->id,
                ];
            }
        }

        if ($deletes) {
            foreach ($deletes as $delete) {
                $store = $this->storeRepo->find($delete['id']);
                if (!$store) {
                    continue;
                }

                if (Gate::deny('store', 'delete', $store)) {
                    continue;
                }

                posify_delete_store($store);
            }
        }

        return $this->json($results, 200);
    }
}
