<?php

namespace Posify\Woocommerce\Http\Api\Controllers;

use Posify\Core\DataTransferObjects\EmployeeQueryArgs;
use Posify\Core\Loader;
use Posify\Core\Repositories\EmployeeRepository;
use Posify\Core\Repositories\StoreRepository;
use Posify\Woocommerce\Authorization\Gate;
use Posify\Woocommerce\Http\Common\Controllers\BaseController;
use WP_REST_Request;

class StoreEmployeeController extends BaseController
{
    protected $storeRepo;
    protected $employeeRepo;

    public function __construct()
    {
        $this->storeRepo = Loader::load(StoreRepository::class);
        $this->employeeRepo = Loader::load(EmployeeRepository::class);
    }

    public function index(WP_REST_Request $wpRequest)
    {
        $store = $this->storeRepo->find($wpRequest->get_param('store_id'));

        if (!$store) {
            return $this->notFound(esc_html__('Store not found', 'posify-for-woocommerce'));
        }

        if (Gate::deny('store', 'read', $store)) {
            return $this->forbidden();
        }

        $dataInStore = $store->dataInStore()->get();

        $args = EmployeeQueryArgs::make($wpRequest->get_params());

        $args->store_id = $store->getId();

        $args->setPage(
            $wpRequest->get_param('page') ?? 1
        );

        $args->setPerPage(
            $wpRequest->get_param('per_page') ?? 15
        );

        $pagination = $this->employeeRepo
            ->paginate($args)
            ->through(function ($employee) use ($dataInStore) {
                return [
                    'id' => $employee->getId(),
                    'username' => $employee->getUsername(),
                    'full_name' => $employee->getFullName(),
                    'email' => $employee->getEmail(),
                    'role' => $employee->getRole(),
                    'avatar_url' => $employee->getAvatarUrl(),
                    'enabled' => (bool) $dataInStore->firstWhere('employee_id', $employee->getId())->enabled,
                ];
            })
            ->toArray();

        $response = $this->json($pagination['data']);
        $response->header('X-WP-Total', $pagination['total']);
        $response->header('X-WP-TotalPages', $pagination['last_page']);

        return $response;
    }

    public function create(WP_REST_Request $wpRequest)
    {
        $store = $this->storeRepo->find($wpRequest->get_param('store_id'));

        if (!$store) {
            return $this->notFound(esc_html__('Store not found', 'posify-for-woocommerce'));
        }

        if (Gate::deny('store_employee', 'create', $store)) {
            return $this->forbidden();
        }

        $employee = null;
        $body = $wpRequest->get_params();
        $employeeId = $body['id'] ?? null;

        if (!$employeeId) {
            $employee = posify_create_employee($body);
        } else {
            $employee = $this->employeeRepo->find($employeeId);
        }

        if (!$employee) {
            return $this->notFound(esc_html__('Employee not found', 'posify-for-woocommerce'));
        }

        if (is_wp_error($employee)) {
            return $this->jsonError($employee);
        }

        posify_add_employee_to_store($store, $employee, $body['enabled'] ?? true);

        return $this->json([
            'id' => $employee->getId(),
        ], 200);
    }

    public function update(WP_REST_Request $wpRequest)
    {
        $store = $this->storeRepo->find($wpRequest->get_param('store_id'));

        if (!$store) {
            return $this->notFound(esc_html__('Store not found', 'posify-for-woocommerce'));
        }

        if (Gate::deny('store_employee', 'update', $store)) {
            return $this->forbidden();
        }

        $employee = $this->employeeRepo->find($wpRequest->get_param('id'));

        if (!$employee) {
            return $this->notFound(esc_html__('Employee not found', 'posify-for-woocommerce'));
        }

        $body = $wpRequest->get_params();
        posify_update_employee_in_store($store, $employee, $body['enabled'] ?? true);

        return $this->json([
            //
        ], 200);
    }

    public function delete(WP_REST_Request $wpRequest)
    {
        $store = $this->storeRepo->find($wpRequest->get_param('store_id'));

        if (!$store) {
            return $this->notFound(esc_html__('Store not found', 'posify-for-woocommerce'));
        }

        if (Gate::deny('store_employee', 'delete', $store)) {
            return $this->forbidden();
        }

        $employee = $this->employeeRepo->find($wpRequest->get_param('id'));

        if (!$employee) {
            return $this->notFound(esc_html__('Employee not found', 'posify-for-woocommerce'));
        }

        posify_remove_employee_from_store($store, $employee);

        return $this->json([], 200);
    }

    public function batch(WP_REST_Request $wpRequest)
    {
        $updates = $wpRequest->get_param('update');
        $deletes = $wpRequest->get_param('delete');
        $creates = $wpRequest->get_param('create');
        $store = $this->storeRepo->find($wpRequest->get_param('store_id'));

        if (!$store) {
            return $this->notFound(esc_html__('Store not found', 'posify-for-woocommerce'));
        }

        $results = [];

        if ($creates && Gate::check('store_employee', 'create', $store)) {
            foreach ($creates as $create) {
                $employee = $this->employeeRepo->find($create['id']);

                if (!$employee) {
                    continue;
                }

                posify_add_employee_to_store($store, $employee, $create['enabled'] ?? true);
            }
        }

        if ($updates && Gate::check('store_employee', 'update', $store)) {
            foreach ($updates as $update) {
                $employee = $this->employeeRepo->find($update['id']);

                if (!$employee) {
                    continue;
                }

                posify_update_employee_in_store($store, $employee, $update['enabled'] ?? true);
            }
        }

        if ($deletes && Gate::check('store_employee', 'delete', $store)) {
            foreach ($deletes as $delete) {
                $employee = $this->employeeRepo->find($delete['id']);

                if (!$employee) {
                    continue;
                }

                posify_remove_employee_from_store($store, $employee);
            }
        }

        return $this->json($results, 200);
    }
}
