<?php

namespace Posify\Woocommerce\Plugins;

use Posify\Core\DataTransferObjects\StockQueryArgs;
use Posify\Core\DataTransferObjects\StoreQueryArgs;
use Posify\Core\Loader;
use Posify\Core\Repositories\SettingRepository;
use Posify\Core\Repositories\StockRepository;
use Posify\Core\Repositories\StoreRepository;
use Posify\Woocommerce\Authorization\Auth;
use Posify\Woocommerce\ServiceProviders\BaseServiceProvider;
use Posify\Woocommerce\Utilities\HasInstance;

class MultipleStockManagement extends BaseServiceProvider
{
    use HasInstance;

    private $settingRepo;
    private $storeRepo;
    private $stockRepo;

    public function __construct()
    {
        $this->settingRepo = Loader::load(SettingRepository::class);
        $this->storeRepo = Loader::load(StoreRepository::class);
        $this->stockRepo = Loader::load(StockRepository::class);
    }

    public function register()
    {
        add_action('woocommerce_product_options_stock_status', [$this, 'showForm'], 10, 3);
        add_action('woocommerce_variation_options_inventory', [$this, 'showForm'], 10, 3);
        add_action('woocommerce_admin_process_product_object', [$this, 'handleUpdateForm'], 10, 3);

        add_action('woocommerce_save_product_variation', function ($variantId, $index) {
            $variant = wc_get_product($variantId);
            return $this->handleUpdateForm($variant, $index);
        }, 10, 3);

        add_filter('woocommerce_can_reduce_order_stock', [$this, 'decreaseStockQuantity'], 100, 2);
        add_filter('woocommerce_can_restore_order_stock', [$this, 'incrementStockQuantity'], 100, 2);
        add_filter('woocommerce_can_restock_refunded_items', [$this, 'restockItemOnRefund'], 10, 3);

        add_filter('woocommerce_rest_prepare_product_object', [$this, 'extendProductRest'], 10, 3);
        add_filter('woocommerce_rest_prepare_product_variation_object', [$this, 'extendProductRest'], 10, 3);
    }

    public function showForm($variantIndex = null, $vairantData = null, $variant = null)
    {
        $target = $variant ? 'variant' : 'product';

        $multipleStockEnabled = $this->settingRepo->get('multistock_enabled');

        if (!$multipleStockEnabled) {
            return;
        }

        global $thepostid;

        $storeQueryArgs = StoreQueryArgs::make([
            'enabled' => true
        ]);

        $storeQueryArgs->setSelect(['id', 'name']);
        $stores = $this->storeRepo->all($storeQueryArgs);

        $stockQueryArgs = StockQueryArgs::make([
            'product_id' => $thepostid,
            'variant_id' => $target == 'variant' ? $variant->ID : 'is_null',
        ]);

        $stockQueryArgs->setSelect(['id', 'store_id', 'stock_quantity']);

        $stocks = $this->stockRepo->all($stockQueryArgs)
            ->transform(function ($stock) use ($stores) {
                $store = $stores->where('id', $stock->store_id)->first();
                $storeName = $store ? $store->name : '';

                return [
                    'id' => $stock->id,
                    'store_id' => $stock->store_id,
                    'store_name' => $storeName,
                    'stock_quantity' => $stock->stock_quantity,
                ];
            });

        $el = 'posify-stock-' . $variantIndex;

        if ($target == 'product') {
            $el = 'posify-stock-management';

            /**
             * ID of the checkbox that triggers the stock management.
             */
            $triggerEl = '#_manage_stock';
        } else {
            $el = 'posify-stock-management-' . $variantIndex;

            /**
             * Name of the checkbox that triggers the stock management for a variant.
             */
            $triggerEl = "input[name='variable_manage_stock[" . $variantIndex . "]']";
        }

        $data = [
            'stores' => $stores,
            'stocks' => $stocks,
            'el' => $el,
            'index' => $variantIndex,
            'triggerEl' => $triggerEl,
        ];

        posify_view('admin/product/pos-stock-management', $data);
    }

    public function handleUpdateForm($product, $variantIndex = null)
    {
        if (
            !Auth::can('manage_woocommerce')
            && !Auth::can('edit_product', $product->get_id())
        ) {
            return;
        }

        if (!isset($_POST['posify_edit_stock']) || !wp_verify_nonce($_POST['posify_edit_stock'], 'posify_edit_stock')) {
            return;
        }

        if (!isset($_POST['posify_stock'])) {
            return;
        }

        $target = $variantIndex !== null ? 'variant' : 'product';
        $multipleStockEnabled = $this->settingRepo->get('multistock_enabled');

        if (!$multipleStockEnabled) {
            return;
        }

        $stocksBody = null;

        /**
         * Sanitize data
         */
        if ($target == 'variant') {
            if (isset($_POST['posify_stock'][$variantIndex])) {
                foreach ($_POST['posify_stock'][$variantIndex] as $item) {
                    $stocksBody[] = [
                        'store_id' => (int) $item['store_id'],
                        'stock_quantity' => (int) $item['stock_quantity'],
                    ];
                }
            }
        } else {
            foreach ($_POST['posify_stock'] as $item) {
                $stocksBody[] = [
                    'store_id' => (int) $item['store_id'],
                    'stock_quantity' => (int) $item['stock_quantity'],
                ];
            }
        }

        if (!$stocksBody) {
            return;
        }

        $stockArgs = StockQueryArgs::make([
            'store_id_not_in' => (array) array_column($stocksBody, 'store_id'),
        ]);

        if ($target == 'variant') {
            $stockArgs->product_id = $product->get_parent_id();
            $stockArgs->variant_id = $product->get_id();
        } else {
            $stockArgs->product_id = $product->get_id();
            $stockArgs->variant_id = 'is_null';
        }

        /**
         * Delete stock that is not in the request.
         *
         * That means the stock is deleted from the UI.
         */
        $this->stockRepo->delete($stockArgs);

        /**
         * Update or create stock.
         */
        foreach ($stocksBody as $stockBody) {
            $storeId = $stockBody['store_id'] ?? null;
            $stockQuantity = $stockBody['stock_quantity'] ?? 0;

            if (!$storeId) {
                continue;
            }

            $store = $this->storeRepo->find($storeId);

            if (!$store) {
                continue;
            }

            $stockArgs = StockQueryArgs::make([
                'store_id' => $store->id,
            ]);

            if ($target == 'variant') {
                $stockArgs->variant_id = $product->get_id();
            } else {
                $stockArgs->product_id = $product->get_id();
                $stockArgs->variant_id = 'is_null';
            }

            $stock = $this->stockRepo->findWhere($stockArgs);

            if ($stock) {
                posify_update_stock($stock, [
                    'stock_quantity' => $stockQuantity
                ]);
            } else {
                $stockData = [
                    'store_id' => $store->id,
                    'stock_quantity' => $stockQuantity
                ];

                if ($target == 'variant') {
                    $stockData['product_id'] = $product->get_parent_id();
                    $stockData['variant_id'] = $product->get_id();
                } else {
                    $stockData['product_id'] = $product->get_id();
                }

                posify_create_stock($stockData);
            }
        }
    }

    public function decreaseStockQuantity($response, $order)
    {
        if ($response) {
            if (posify_is_pos_order($order)) {
                $storeId = $order->get_meta('_posify_store_id');

                if (!$storeId) {
                    return false;
                }

                $store = $this->storeRepo->find($storeId);

                if (!$store) {
                    return false;
                }

                $this->reduceStockLevels($order, $store);
            }
        }

        return $response;
    }

    private function reduceStockLevels($order, $store)
    {
        if (!$order || 'yes' !== get_option('woocommerce_manage_stock') || !apply_filters('posify_can_reduce_order_stock', true, $order)) {
            return;
        }

        $changes = [];

        $multipleStockEnabled = $this->settingRepo->get('multistock_enabled');
        $multiStockCondition = $this->settingRepo->get('multistock_condition');

        foreach ($order->get_items() as $item) {
            if (!$item->is_type('line_item')) {
                continue;
            }

            $product = $item->get_product();
            $itemStockReduced = $item->get_meta('_reduced_stock', true);

            if ($itemStockReduced || !$product || !$product->managing_stock()) {
                continue;
            }

            $quantity = apply_filters('woocommerce_order_item_quantity', $item->get_quantity(), $order, $item);
            $productIdWithStock = $product->get_stock_managed_by_id();
            $product = $productIdWithStock !== $product->get_id() ? wc_get_product($productIdWithStock) : $product;
            $newStock = false;
            $reducedBy = false;

            $stockArgs = StockQueryArgs::make([
                'store_id' => $store->id,
            ]);

            if ($product->is_type('variable')) {
                $stockArgs->variation_id = $product->get_id();
                $stockArgs->product_id = $product->get_parent_id();
            } else {
                $stockArgs->product_id = $product->get_id();
                $stockArgs->variant_id = 'is_null';
            }

            $stock = $this->stockRepo->findWhere($stockArgs);

            if ($stock) {
                $newStock = posify_decrease_stock_quantity($stock, $quantity);
                $reducedBy = 'store';
            } elseif (!$multipleStockEnabled || $multiStockCondition == 'general') {
                $newStock = wc_update_product_stock($product, $quantity, 'decrease');
                $reducedBy = 'general';
            }

            if (is_wp_error($newStock)) {
                $itemName = $product->get_formatted_name();

                /* translators: %s: product name */
                $order->add_order_note(sprintf(esc_html__('Unable to reduce stock for item %s.', 'woocommerce'), $itemName));
                continue;
            }

            if ($newStock) {
                $item->add_meta_data('_reduced_stock', $quantity, true);

                if ($reducedBy == 'store') {
                    $item->add_meta_data('_posify_reduced_stock_by_store', $store->id, true);
                    $item->add_meta_data('_posify_reduced_stock_by_store_quantity', $quantity, true);
                }

                if ($reducedBy == 'store') {
                    $item->add_meta_data('_posify_reduced_stock_by_general', $quantity, true);
                }

                $item->save();
            }

            $changes[] = [
                'product' => $product,
                'from'    => $newStock + $quantity,
                'to'      => $newStock,
            ];
        }

        if ($changes) {
            wc_trigger_stock_change_notifications($order, $changes);
            do_action('posify_reduce_order_stock', $order);
        }
    }

    public function incrementStockQuantity($response, $order)
    {
        if ($response) {
            if (posify_is_pos_order($order)) {
                $storeId = $order->get_meta('_posify_store_id');
                if (!$storeId) {
                    return false;
                }

                $store = $this->storeRepo->find($storeId);

                if (!$store) {
                    return false;
                }

                $this->restoreStockLevels($order, $store);
            }
        }

        return $response;
    }

    private function restoreStockLevels($order, $store)
    {
        if (!$order || 'yes' !== get_option('woocommerce_manage_stock') || !apply_filters('posify_can_reduce_order_stock', true, $order)) {
            return;
        }

        $changes = [];

        $multipleStockEnabled = $this->settingRepo->get('multistock_enabled');
        $multiStockCondition = $this->settingRepo->get('multistock_condition');

        foreach ($order->get_items() as $item) {
            if (!$item->is_type('line_item')) {
                continue;
            }

            $product = $item->get_product();
            $itemStockReduced = $item->get_meta('_reduced_stock', true);

            if (!$product) {
                continue;
            }

            $productIdWithStock = $product->get_stock_managed_by_id();
            $product = $productIdWithStock !== $product->get_id() ? wc_get_product($productIdWithStock) : $product;

            if (!$product || !$itemStockReduced || !$product->managing_stock()) {
                continue;
            }

            $newStock = false;
            $restoreIn = false;

            $stockArgs = StockQueryArgs::make([
                'store_id' => $store->id,
            ]);

            if ($product->is_type('variable')) {
                $stockArgs->variation_id = $product->get_id();
                $stockArgs->product_id = $product->get_parent_id();
            } else {
                $stockArgs->product_id = $product->get_id();
                $stockArgs->variant_id = 'is_null';
            }

            $stock = $this->stockRepo->findWhere($stockArgs);

            if ($stock) {
                $newStock = posify_increase_stock_quantity($stock,  $itemStockReduced);
                $restoreIn = 'store';
            } elseif (!$multipleStockEnabled || $multiStockCondition == 'general') {
                $newStock = wc_update_product_stock($product, $itemStockReduced, 'increase');
                $restoreIn = 'general';
            }

            $itemName = $product->get_formatted_name();

            if (is_wp_error($newStock)) {
                /* translators: %s: product name */
                $order->add_order_note(sprintf(esc_html__('Unable to reduce stock for item %s.', 'woocommerce'), $itemName));
                continue;
            }

            $item->delete_meta_data('_reduced_stock');

            if ($restoreIn == 'store') {
                $item->delete_meta_data('_posify_reduced_stock_by_store');
                $item->delete_meta_data('_posify_reduced_stock_by_store_quantity');
            }

            if ($restoreIn == 'store') {
                $item->delete_meta_data('_posify_reduced_stock_by_general');
            }

            $item->save();

            $changes[] = $itemName . ' ' . ($newStock - $itemStockReduced) . '&rarr;' . $newStock;
        }

        if ($changes) {
            $order->add_order_note(esc_html__('Stock levels increased:', 'woocommerce') . ' ' . implode(', ', $changes));
        }

        do_action('posify_restore_order_stock', $order);
    }

    public function restockItemOnRefund($allowed, $order, $refundedLineItem)
    {
        if (!posify_is_pos_order($order)) {
            return $allowed;
        }

        $allowed = false;
        $storeId = $order->get_meta('_posify_store_id');

        if (!$storeId) {
            return $allowed;
        }

        $lineItems = $order->get_items();

        foreach ($lineItems as $itemId => $item) {
            if (!isset($refundedLineItem[$itemId], $refundedLineItem[$itemId]['qty'])) {
                continue;
            }

            $product = $item->get_product();
            $itemStockReduced = $item->get_meta('_reduced_stock', true);
            $quantityToRefund = $refundedLineItem[$itemId]['qty'];

            if (!$itemStockReduced || !$quantityToRefund || !$product || !$product->managing_stock()) {
                continue;
            }

            $stockArgs = StockQueryArgs::make([
                'store_id' => $storeId,
            ]);

            if ($product->is_type('variable')) {
                $stockArgs->variation_id = $product->get_id();
                $stockArgs->product_id = $product->get_parent_id();
            } else {
                $stockArgs->product_id = $product->get_id();
                $stockArgs->variant_id = 'is_null';
            }

            $stock = $this->stockRepo->findWhere($stockArgs);

            $newStock = false;
            $restoredIn = false;

            if ($stock && $storeId == $item->get_meta('_posify_reduced_stock_by_store')) {
                $newStock = posify_increase_stock_quantity($stock,  $itemStockReduced);
                $restoredIn = 'store';
            } elseif ($item->get_meta('_posify_reduced_stock_by_general')) {
                $oldStock = $product->get_stock_quantity();
                $newStock = wc_update_product_stock($product, $quantityToRefund, 'increase');
                $restoredIn = 'general';
            }

            $itemStockReduced = $itemStockReduced - $quantityToRefund;

            if (0 < $itemStockReduced) {
                $item->update_meta_data('_reduced_stock', $itemStockReduced);

                if ($restoredIn == 'store') {
                    $item->update_meta_data('_posify_reduced_stock_by_store_quantity', $itemStockReduced);
                } elseif ($restoredIn == 'general') {
                    $item->update_meta_data('_posify_reduced_stock_by_general_quantity', $itemStockReduced);
                }
            } else {
                $item->delete_meta_data('_reduced_stock');

                if ($restoredIn == 'store') {
                    $item->delete_meta_data('_posify_reduced_stock_by_store');
                    $item->delete_meta_data('_posify_reduced_stock_by_store_quantity');
                } elseif ($restoredIn == 'general') {
                    $item->delete_meta_data('_posify_reduced_stock_by_general');
                }
            }

            /* translators: %1$s: product id, %2$s: old stock, %3$s: new stock */
            $order->add_order_note(sprintf(esc_html__('Item #%1$s stock increased from %2$s to %3$s.', 'woocommerce'), $product->get_id(), $oldStock, $newStock));
            $item->save();

            do_action('woocommerce_restock_refunded_item', $product->get_id(), $oldStock, $newStock, $order, $product);
        }
    }

    public function extendProductRest($response, $product, $request)
    {
        $multipleStockEnabled = $this->settingRepo->get('multistock_enabled');

        if (!$multipleStockEnabled || !$product->managing_stock()) {
            return $response;
        }

        $storeId = $request->get_param('posify_store');

        if (!$storeId) {
            return $response;
        }

        $stockArgs = StockQueryArgs::make([
            'store_id' => $storeId,
        ]);

        if ($product->is_type('variable')) {
            $stockArgs->variation_id = $product->get_id();
            $stockArgs->product_id = $product->get_parent_id();
        } else {
            $stockArgs->product_id = $product->get_id();
            $stockArgs->variant_id = 'is_null';
        }

        $stockRepository = Loader::load(StockRepository::class);
        $stock = $stockRepository->findWhere($stockArgs);

        $stockQuantity = $response->data['stock_quantity'];
        $stockStatus = $response->data['stock_status'];
        $manageStock = $product->managing_stock();

        if ($stock) {
            $stockQuantity = $stock->getStockQuantity();
            $stockStatus = $stock->getStockStatus();
            $manageStock = true;
        }

        if (!$stock) {
            $multiStockCondition = $this->settingRepo->get('multistock_condition');
            if ($multiStockCondition == 'general') {
                $stockQuantity = $response->data['stock_quantity'];
                $stockStatus = $response->data['stock_status'];
                $manageStock = true;
            }

            if ($multiStockCondition == 'allowed') {
                $stockStatus = 'instock';
                $stockQuantity = 0;
                $manageStock = false;
            }

            if ($multiStockCondition == 'not_allowed') {
                $stockStatus = 'outofstock';
                $stockQuantity = 0;
                $manageStock = true;
            }
        }

        /**
         * Override the stock quantity and status.
         */
        $response->data['posify_stock_quantity'] = $stockQuantity;
        $response->data['posify_stock_status'] = $stockStatus;
        $response->data['posify_manage_stock'] = $manageStock;

        /**
         * Override the purchasable status.
         */
        if (isset($response->data['posify_purchasable']) && $response->data['posify_purchasable']) {
            if ($stockStatus == 'outofstock') {
                $response->data['posify_purchasable'] = false;
            }
        }

        return $response;
    }
}
