<?php

namespace Posify\Woocommerce\Reports;

use Carbon\Carbon;
use Posify\Core\DataTransferObjects\IntervalReportQuery;
use Posify\Core\DataTransferObjects\OverviewReportQuery;
use Posify\Core\Reports\AbstractIntervalReport;
use Posify\Woocommerce\Reports\OverviewReport;

class IntervalReport extends AbstractIntervalReport
{
    protected $maxInterval = '24';

    public IntervalReportQuery $query;

    public function __construct(
        IntervalReportQuery $query
    ) {
        $this->query = $query;
    }

    public static function make(IntervalReportQuery $query): self
    {
        return new self($query);
    }

    public function toArray()
    {
        $intervalQuery = $this->getIntervalQuery();
        $results = [];

        foreach ($intervalQuery as $query) {
            $report = OverviewReport::make(OverviewReportQuery::make([
                'start_time' => $query['start_time'],
                'end_time' => $query['end_time'],
                'store_id' => $this->query->store_id,
                'register_id' => $this->query->register_id,
                'employee_id' => $this->query->employee_id,
            ]));

            $report->calculate();

            $results[] = [
                'interval' => $query['interval'],
                'total_sales' => $report->getTotalSales(),
                'total_items' => $report->getTotalItems(),
                'total_discounts' => $report->getTotalDiscounts(),
                'total_orders' => $report->getTotalOrders(),
                'total_customers' => $report->getTotalCustomers(),
                'total_net_sales' => $report->getTotalNetSales(),
                'avg_order_value' => $report->getAvgOrderValue(),
                'avg_items_per_order' => $report->getAvgItemsPerOrder(),
            ];
        }

        return $results;
    }

    private function getIntervalQuery(): array
    {
        $startTime = Carbon::createFromFormat('Y-m-d H:i:s', $this->query->start_time);
        $endTime = Carbon::createFromFormat('Y-m-d H:i:s', $this->query->end_time);
        $interval = $this->findInterval($startTime, $endTime);

        $intervalQuery = [];

        if ($interval == 'hour') {
            $hours = $startTime->diffInHours($endTime);
            foreach (range(0, $hours) as $hour) {
                $baseTime = $startTime->clone()->addHours($hour);
                $intervalQuery[] = [
                    'start_time' => $baseTime->startOfHour()->format('Y-m-d H:i:s'),
                    'end_time' => $baseTime->endOfHour()->format('Y-m-d H:i:s'),
                    'interval' => $baseTime->format('Y-m-d H'),
                ];
            }
        }

        if ($interval == 'day') {
            $days = $startTime->diffInDays($endTime);
            foreach (range(0, $days) as $day) {
                $baseTime = $startTime->clone()->addDays($day);
                $intervalQuery[] = [
                    'start_time' => $baseTime->startOfDay()->format('Y-m-d H:i:s'),
                    'end_time' => $baseTime->endOfDay()->format('Y-m-d H:i:s'),
                    'interval' => $baseTime->format('Y-m-d'),
                ];
            }
        }

        if ($interval == 'week') {
            $weeks = $startTime->diffInWeeks($endTime);
            foreach (range(0, $weeks) as $week) {
                $baseTime = $startTime->clone()->addWeeks($week);

                $intervalQuery[] = [
                    'start_time' => $baseTime->startOfWeek()->format('Y-m-d H:i:s'),
                    'end_time' => $baseTime->endOfWeek()->format('Y-m-d H:i:s'),
                    'interval' => $baseTime->format('Y-m-d'),
                ];
            }
        }

        if ($interval == 'month') {
            $months = $startTime->diffInMonths($endTime);
            foreach (range(0, $months) as $month) {
                $baseTime = $startTime->clone()->addMonths($month);

                $intervalQuery[] = [
                    'start_time' => $baseTime->startOfMonth()->format('Y-m-d H:i:s'),
                    'end_time' => $baseTime->endOfMonth()->format('Y-m-d H:i:s'),
                    'interval' => $baseTime->format('Y-m'),
                ];
            }
        }

        if ($interval == 'quater') {
            $quaters = $startTime->diffInQuarters($endTime);
            foreach (range(0, $quaters) as $quater) {
                $baseTime = $startTime->clone()->addQuarters($quater);

                $intervalQuery[] = [
                    'start_time' => $baseTime->startOfQuarter()->format('Y-m-d H:i:s'),
                    'end_time' => $baseTime->endOfQuarter()->format('Y-m-d H:i:s'),
                    'interval' => $baseTime->format('Y-m'),
                ];
            }
        }

        if ($interval == 'year') {
            $years = $startTime->diffInYears($endTime);
            foreach (range(0, $years) as $year) {
                $baseTime = $startTime->clone()->addYears($year);

                $intervalQuery[] = [
                    'start_time' => $baseTime->startOfYear()->format('Y-m-d H:i:s'),
                    'end_time' => $baseTime->endOfYear()->format('Y-m-d H:i:s'),
                    'interval' => $baseTime->format('Y'),
                ];
            }
        }

        return $intervalQuery;
    }

    /**
     * Find interval
     *
     * @param Carbon $startTime
     * @param Carbon $endTime
     *
     * @return string
     */
    private function findInterval($startTime, $endTime): string
    {
        $diffInHours = $startTime->diffInHours($endTime);

        if ($diffInHours <= $this->maxInterval) {
            return 'hour';
        }

        $diffInDays = $startTime->diffInDays($endTime);
        if ($diffInDays <= $this->maxInterval) {
            return 'day';
        }

        $diffInWeeks = $startTime->diffInWeeks($endTime);
        if ($diffInWeeks <= $this->maxInterval) {
            return 'week';
        }

        $diffInMonths = $startTime->diffInMonths($endTime);
        if ($diffInMonths <= $this->maxInterval) {
            return 'month';
        }

        $diffInQuarters = $startTime->diffInQuarters($endTime);
        if ($diffInQuarters <= $this->maxInterval) {
            return 'quater';
        }

        return 'year';
    }
}
