<?php

namespace Posify\Woocommerce\Reports;

use Posify\Core\Entities\Employee;
use Posify\Core\Entities\RegisterSession;
use Posify\Core\Reports\AbstractRegisterSessionReport;
use Posify\Core\Utilities\HasMakeFactory;
use Posify\Woocommerce\DB;
use Posify\Woocommerce\Entities\Order;
use Posify\Woocommerce\Entities\RegisterCash;
use Posify\Woocommerce\Payment\PosifyCashPaymentGateway;

class RegisterSessionReport extends AbstractRegisterSessionReport
{
    use HasMakeFactory;

    public RegisterSession $session;
    public ?Employee $employee = null;

    public function __construct(
        RegisterSession $session,
        ?Employee $employee = null
    ) {
        $this->session = $session;
        $this->employee = $employee;
    }

    public function getOrdersCount()
    {
        return Order::where('register_session_id', $this->session->getId())
            ->when($this->employee, function ($query) {
                $query->where('cashier_id', $this->employee->getId());
            })
            ->count();
    }

    public function getNumItemsSold()
    {
        return Order::where('register_session_id', $this->session->getId())
            ->when($this->employee, function ($query) {
                $query->where('cashier_id', $this->employee->getId());
            })
            ->sum('total_items');
    }

    /**
     * Return only cash in hand remaining
     */
    public function getCashInHand()
    {
        return RegisterCash::where('session_id', $this->session->getId())
            ->when($this->employee, function ($query) {
                $query->where('employee_id', $this->employee->getId());
            })
            ->sum('amount');
    }

    public function getCashChanges()
    {
        return RegisterCash::where('session_id', $this->session->getId())
            ->when($this->employee, function ($query) {
                $query->where('employee_id', $this->employee->getId());
            })
            ->orderBy('id', 'asc')
            ->get()
            ->toArray();
    }

    /**
     * Return only values from cash payment gateway
     */
    public function getCash()
    {
        return Order::join('posify_order_payment_gateways', 'posify_order_payment_gateways.order_id', '=', 'posify_orders.id')
            ->where('posify_orders.register_session_id', $this->session->getId())
            ->where('posify_order_payment_gateways.payment_gateway_id', PosifyCashPaymentGateway::ID)
            ->when($this->employee, function ($query) {
                $query->where('posify_orders.cashier_id', $this->employee->getId());
            })
            ->sum('posify_order_payment_gateways.amount');
    }

    /**
     * Return total cash in hand
     * Include cash payments and cash in hand
     */
    public function getCashTotal()
    {
        return $this->getCash() + $this->getCashInHand();
    }

    public function getNetTotal()
    {
        return $this->getTotalSales() - $this->getTotalDiscounts();
    }

    public function getTotalSales()
    {
        return Order::where('register_session_id', $this->session->getId())
            ->when($this->employee, function ($query) {
                $query->where('cashier_id', $this->employee->getId());
            })
            ->sum('order_total');
    }

    public function getTotalDiscounts()
    {
        return Order::where('register_session_id', $this->session->getId())
            ->when($this->employee, function ($query) {
                $query->where('cashier_id', $this->employee->getId());
            })
            ->sum('cart_discount');
    }

    public function getOpenTime()
    {
        return $this->session->getOpenTime();
    }

    public function getCashiers(): array
    {
        return $this->session->employees->map(function ($employee) {
            return [
                'id' => $employee->getId(),
                'username' => $employee->getUsername(),
                'full_name' => $employee->getFullname(),
                'role' => $employee->getRole(),
                'avatar_url' => $employee->getAvatarUrl(),
            ];
        })->toArray();
    }

    public function getCashierActions(): array
    {
        return $this->session->employeeActions->map(function ($action) {
            return [
                'id' => $action->getId(),
                'employee_id' => $action->getEmployeeId(),
                'action' => $action->getAction(),
                'created_at' => $action->getCreatedAt()->toDateTimeString(),
                'meta' => $action->getMeta(),
            ];
        })->toArray();
    }

    /**
     * List all payment gateways in this session
     */
    public function getPaymentGateways(): array
    {
        return Order::join('posify_order_payment_gateways', 'posify_order_payment_gateways.order_id', '=', 'posify_orders.id')
            ->where('posify_orders.register_session_id', $this->session->getId())
            ->when($this->employee, function ($query) {
                $query->where('posify_orders.cashier_id', $this->employee->getId());
            })
            ->select('posify_order_payment_gateways.payment_gateway_id', 'posify_order_payment_gateways.payment_gateway_title')
            ->distinct()
            ->get()
            ->toArray();
    }

    public function getPaymentGatewaysWithTotalSale()
    {
        $prefix = DB::getTablePrefix();

        return Order::join('posify_order_payment_gateways', 'posify_order_payment_gateways.order_id', '=', 'posify_orders.id')
            ->select([
                'posify_order_payment_gateways.payment_gateway_id',
                DB::raw("SUM({$prefix}posify_order_payment_gateways.amount) as total_sales"),
            ])
            ->where('posify_orders.register_session_id', $this->session->getId())
            ->when($this->employee, function ($query) {
                $query->where('posify_orders.cashier_id', $this->employee->getId());
            })
            ->groupBy('posify_order_payment_gateways.payment_gateway_id')
            ->get()
            ->map(function ($item) {
                return [
                    'id' => $item->payment_gateway_id,
                    'title' => posify_wc_get_payment_gateway_title_by_id($item->payment_gateway_id),
                    'total_sales' => $item->total_sales,
                ];
            })
            ->toArray();
    }

    public function toArray()
    {
        return [
            'id' => $this->session->getId(),
            'store_id' => $this->session->getStoreId(),
            'register_id' => $this->session->getRegisterId(),
            'open_time' => $this->getOpenTime(),
            'close_time' => $this->session->getCloseTime(),
            'orders_count' => $this->getOrdersCount(),
            'num_items_sold' => $this->getNumItemsSold(),
            'total_sales' => $this->getTotalSales(),
            'total_discounts' => $this->getTotalDiscounts(),
            'net_total' => $this->getNetTotal(),
            'cash' => $this->getCash(),
            'cash_in_hand' => $this->getCashInHand(),
            'cash_total' => $this->getCashTotal(),
            'cashiers' => $this->getCashiers(),
            'cashier_actions' => $this->getCashierActions(),
            'payment_gateways' => $this->getPaymentGatewaysWithTotalSale(),
            'cash_changes' => $this->getCashChanges(),
        ];
    }
}
