<?php

namespace Posify\Woocommerce\Repositories;

use Posify\Core\DataTransferObjects\RegisterQueryArgs;
use Posify\Core\Entities\Receipt;
use Posify\Core\Entities\Store;
use Posify\Core\Repositories\RegisterRepository as CoreRegisterRepository;
use Posify\Core\ValueObjects\Undefined;
use Posify\Woocommerce\Entities\Register;
use Posify\Woocommerce\Payment\PosifyCashPaymentGateway;

class RegisterRepository extends EloquentRepository implements CoreRegisterRepository
{
    public function query(RegisterQueryArgs $args)
    {
        return Register::query()
            ->when($args->keyword, function ($query) use ($args) {
                $query->where('name', 'LIKE', $args->keyword . '%');
            })
            ->when($args->getWith(), function ($query) use ($args) {
                $query->with($args->getWith());
            })
            ->when($args->getSort(), function ($query) use ($args) {
                $query->orderBy($args->getSort(), $args->getOrder());
            })
            ->when($args->id, function ($query) use ($args) {
                $query->where('id', $args->id);
            })
            ->when($args->store_id, function ($query) use ($args) {
                $query->where('store_id', $args->store_id);
            })
            ->when($args->store_has_employee_id, function ($query) use ($args) {
                $query->whereHas('store', function ($query) use ($args) {
                    $query->whereHas('employees', function ($query) use ($args) {
                        $query->where('users.ID', $args->store_has_employee_id);
                    });
                });
            })
            ->when(Undefined::isNot($args->enabled), function ($query) use ($args) {
                $query->where('enabled', $args->enabled);
            });
    }

    public function paginate(RegisterQueryArgs $args)
    {
        return $this
            ->query($args)
            ->paginate($args->getPerpage(), $args->getSelect(), $args->getPageParam(), $args->getPage());
    }

    public function find(int $id): ?Register
    {
        return Register::find($id);
    }

    public function exists(int $id): bool
    {
        return Register::where('id', $id)->exists();
    }

    public function findOrCreateDefaultRegister(Store $store = null, Receipt $receipt = null): Register
    {
        $register = Register::query()->orderBy('id', 'asc')->first();

        if (!$register) {
            $register = posify_create_register([
                'enabled' => true,
                'name' => esc_html__('Default Register', 'posify-for-woocommerce'),
                'store_id' => $store ? $store->getId() : null,
                'receipt_id' => $receipt ? $receipt->getId() : null,
                'shipping_methods' => [
                    'flat_rate',
                    'free_shipping',
                    'local_pickup',
                ],

                'payment_gateways' => [
                    PosifyCashPaymentGateway::ID,
                ],

                'default_payment_gateway' => PosifyCashPaymentGateway::ID,
                'default_shipping_method' => 'flat_rate',
            ]);
        }

        return $register;
    }

    public function count(RegisterQueryArgs $args): int
    {
        return $this->query($args)->count();
    }

    public function findWhere(RegisterQueryArgs $args): ?Register
    {
        return $this->query($args)->first();
    }
}
