<?php

namespace Posify\Woocommerce\ServiceProviders;

use Posify\Woocommerce\Http\Web\Controllers\AdminController;
use Posify\Woocommerce\ServiceProviders\BaseServiceProvider;
use Posify\Woocommerce\Utilities\HasInstance;

class AdminServiceProvider extends BaseServiceProvider
{
    use HasInstance;

    public function register()
    {
        $this->registerAdminMenu();
        $this->registerAdminScripts();
        $this->enqueueAdminScriptsForProVersion();
        $this->enqueueAdminScriptsForFreeVersion();
        $this->loadScriptTagModule();
        $this->registerPosOrderMetaBox();
        $this->registerFilterOrderSourceForm();
        $this->registerFilterOrderSource();
    }

    protected function registerAdminMenu()
    {
        add_action('admin_menu', function () {
            // phpcs:ignore WordPress.WP.AlternativeFunctions.file_get_contents_file_get_contents
            $svgIcon = 'data:image/svg+xml;base64,' . base64_encode(file_get_contents(posify_plugin_dist_path('/pos/logo.svg')));
            add_menu_page(
                __('Posify Woocommerce', 'posify-for-woocommerce'),
                'Posify',
                'posify_manage_store',
                'posify-for-woocommerce',
                [new AdminController(), 'index'],
                $svgIcon
            );
        });
    }

    protected function registerAdminScripts()
    {
        add_action('admin_enqueue_scripts', function () {
            $distBaseUrl = posify_plugin_dist_url();

            /**
             * Common scripts
             */
            wp_register_style('posify_admin_fix_wp_styles', $distBaseUrl . '/custom/admin-fix-wp.css', null, POSIFY_VERSION);
            wp_register_style('posify_admin_custom_styles', $distBaseUrl . '/custom/admin.css', null, POSIFY_VERSION);

            /**
             * Pro version scripts
             */
            wp_register_script('posify_admin_pro_version_main_scripts', $distBaseUrl . '/admin/pro-version/app/main.js', null, POSIFY_VERSION, true);
            wp_register_script('posify_admin_pro_version_runtime_dom_scripts', $distBaseUrl . '/admin/pro-version/app/runtime-dom.esm-bundler.js', null, POSIFY_VERSION, true);
            wp_register_style('posify_admin_pro_version_main_styles', $distBaseUrl . '/admin/pro-version/app/index.css', null, POSIFY_VERSION);
            wp_register_script('posify_admin_pro_version_product_stock_scripts', $distBaseUrl . '/admin/pro-version/app/stock-management.js', null, POSIFY_VERSION, true);

            /**
             * Free version scripts
             */
            wp_register_script('posify_admin_free_version_main_scripts', $distBaseUrl . '/admin/free-version/app/free-version.js', null, POSIFY_VERSION, true);
            wp_register_style('posify_admin_free_version_main_styles', $distBaseUrl . '/admin/free-version/app/free-version.css', null, POSIFY_VERSION);
        }, 11);
    }

    protected function enqueueAdminScriptsForProVersion()
    {
        if (!posify_is_pro_version()) {
            return;
        }

        add_action('admin_enqueue_scripts', function ($hook) {
            global $post;

            wp_enqueue_style('posify_admin_fix_wp_styles');

            if ('toplevel_page_posify-for-woocommerce' == $hook) {
                wp_enqueue_media();
                wp_enqueue_style('posify_admin_custom_styles');

                wp_enqueue_script('posify_admin_pro_version_main_scripts');
                wp_enqueue_script('posify_admin_pro_version_runtime_dom_scripts');
                wp_enqueue_style('posify_admin_pro_version_main_styles');

                return;
            }

            if ($hook == 'post-new.php' || $hook == 'post.php') {
                if ('product' === $post->post_type) {
                    wp_enqueue_script('posify_admin_pro_version_product_stock_scripts');
                }

                return;
            }
        }, 11);
    }

    protected function enqueueAdminScriptsForFreeVersion()
    {
        if (posify_is_pro_version()) {
            return;
        }

        add_action('admin_enqueue_scripts', function ($hook) {
            wp_enqueue_style('posify_admin_fix_wp_styles');

            if ('toplevel_page_posify-woocommerce' == $hook) {
                wp_enqueue_media();
                wp_enqueue_style('posify_admin_custom_styles');

                wp_enqueue_script('posify_admin_free_version_main_scripts');
                wp_enqueue_style('posify_admin_free_version_main_styles');
            }
        }, 11);
    }

    protected function loadScriptTagModule()
    {
        add_filter('script_loader_tag', function ($tag, $handle, $src) {
            if (in_array($handle, [
                'posify_admin_pro_version_main_scripts',
                'posify_admin_pro_version_runtime_dom_scripts',
                'posify_admin_pro_version_product_stock_scripts',
                'posify_admin_free_version_main_scripts',
            ])) {
                $tag = '<script type="module" crossorigin src="' . esc_url($src) . '"></script>';
            }

            return $tag;
        }, 10, 3);
    }

    protected function registerPosOrderMetaBox()
    {
        add_action('add_meta_boxes', function () {
            // phpcs:ignore WordPress.Security.NonceVerification.Recommended
            $postId = isset($_REQUEST['post']) ? sanitize_key($_REQUEST['post']) : (isset($_REQUEST['post_ID']) ? sanitize_key($_REQUEST['post_ID']) : 0);

            // phpcs:ignore WordPress.Security.NonceVerification.Recommended
            $postType = isset($_GET['post_type']) ? sanitize_key($_GET['post_type']) : null;

            if ($postType && 'shop_order' === $postType || posify_is_shop_order($postId)) {
                $order = wc_get_order($postId);

                if (!$order) {
                    return;
                }

                $isPosOrder = posify_is_pos_order($order);

                if (!$isPosOrder) {
                    return;
                }

                $screen = posify_get_shop_order_screen();

                add_meta_box(
                    'posify-pos-order-meta',
                    esc_html__('POS Order', 'posify-for-woocommerce'),
                    function () use ($order) {
                        $paymentGateways = [];
                        $wcOrderMetaData = $order->get_meta_data();

                        foreach ($wcOrderMetaData as $metaData) {
                            $metaKey = $metaData->key;
                            $metaValue = $metaData->value;

                            if (strpos($metaKey, '_posify_gateway_total_') === 0) {
                                $paymentGatewayId = str_replace('_posify_gateway_total_', '', $metaKey);
                                $paymentGatewayTitle = posify_wc_get_payment_gateway_title_by_id($paymentGatewayId);

                                $paymentGateways[] = [
                                    'payment_gateway_id' => $paymentGatewayId,
                                    'payment_gateway_title' => $paymentGatewayTitle,
                                    'amount' => $metaValue,
                                ];
                            }
                        }

                        $data['storeName'] = $order->get_meta('_posify_store_name');
                        $data['registerName'] = $order->get_meta('_posify_register_name');
                        $data['cashierName'] = $order->get_meta('_posify_cashier_name');
                        $data['cashierId'] = $order->get_meta('_posify_cashier_id');
                        $data['paymentGateways'] = $paymentGateways;
                        $data['currency'] = $order->get_currency();

                        posify_view('admin/order/order-metabox', $data);
                    },
                    $screen,
                    'side',
                    'high'
                );
            }
        });
    }

    protected function registerFilterOrderSourceForm()
    {
        add_action('restrict_manage_posts', function ($postType) {
            if ('shop_order' === $postType) {
                // phpcs:ignore WordPress.Security.NonceVerification.Recommended
                $selectedType = isset($_REQUEST['_posify_source']) ? sanitize_key($_REQUEST['_posify_source']) : '';

                $types = array(
                    'posify' => esc_html__('Posify order', 'posify-for-woocommerce'),
                    'online' => esc_html__('Online order', 'posify-for-woocommerce')
                );

                $placeholder = esc_attr__('Filter order source', 'posify-for-woocommerce');

                $enhancedAttrs = implode(' ', array(
                    "class='wc-enhanced-select'",
                    "data-placeholder='{$placeholder}'",
                    "data-allow_clear='true'",
                    "aria-hidden='true'",
                    "style='min-width:200px;'"
                ));

                echo "<select name='_posify_source' " . esc_html($enhancedAttrs) . ">";
                echo "<option value='' />";

                foreach ($types as $id => $name) {
                    echo "<option value='" . esc_attr($id) . "' " . selected($id, $selectedType, false) . ">" . esc_html($name) . "</option>";
                }

                echo "</select>";
            }
        }, 10, 1);
    }

    protected function registerFilterOrderSource()
    {
        add_action('pre_get_posts', function ($query) {
            if ($query->is_main_query() && isset($query->query['post_type']) && 'shop_order' === $query->query['post_type']) {
                $meta_query = !!$query->get('meta_query') ? $query->get('meta_query') : [];
                $changed = false;

                // phpcs:ignore WordPress.Security.NonceVerification.Recommended
                $source = isset($_REQUEST['_posify_source']) ? sanitize_key($_REQUEST['_posify_source']) : '';

                if ($source) {
                    if ('posify' === $source) {
                        $changed      = true;
                        $meta_query[] = ['key' => '_posify_source', 'value' => 'posify'];
                    } elseif ('online' === $source) {
                        $changed      = true;
                        $meta_query[] = [
                            'relation' => 'OR',
                            ['key' => '_posify_source', 'value' => '0'],
                            ['key' => '_posify_source', 'compare' => 'NOT EXISTS']
                        ];
                    }
                }

                if ($changed) {
                    $query->set('meta_query', $meta_query);
                }
            }
        }, 10, 1);
    }
}
