<?php

namespace Posify\Woocommerce\ServiceProviders;

use Posify\Woocommerce\Enums\EmployeeRoles;
use Posify\Woocommerce\Utilities\HasInstance;

class AuthServiceProvider extends BaseServiceProvider
{
    use HasInstance;

    protected $roleAndPermissionVersion = '1.0.7';

    protected $roleAndPermissionVersionOption = 'posify_roles_and_permissions_version';

    protected function getPosifyPermissions()
    {
        return [
            'posify_manage_pos',
            'posify_manage_store',
            'posify_use_pos',
        ];
    }

    protected function getPosManagerPermissions()
    {
        return [
            'posify_manage_store',
            'posify_only_their_data',
        ];
    }

    protected function getPosCashierPermissions()
    {
        return [
            'posify_use_pos',
            'posify_only_their_data',
            'posify_only_published_data',
        ];
    }

    public function register()
    {
        $this->registerRoles();
        $this->logLastLogin();
    }

    protected function logLastLogin()
    {
        add_action('wp_login', function ($userLogin, $user) {
            update_user_meta($user->ID, 'posify_last_login', current_time('mysql'));
        }, 10, 2);
    }

    protected function registerRoles()
    {
        add_action('init', function () {
            $currentVersion = get_option($this->roleAndPermissionVersionOption, '1.0.0');

            if ($currentVersion != $this->roleAndPermissionVersion) {
                // 1. Remove old roles
                remove_role(EmployeeRoles::CASHIER);
                remove_role(EmployeeRoles::MANAGER);

                // Remove depreciated roles
                remove_role('wooify_cashier');
                remove_role('wooify_manager');
                remove_role('wooify_pos_cashier');
                remove_role('wooify_pos_manager');

                // 2. Register pos cashier role and permissions
                $posCashierPermissions = array_merge(
                    array_keys(get_role('subscriber')->capabilities),
                    $this->getPosCashierPermissions()
                );

                add_role(
                    EmployeeRoles::CASHIER,
                   esc_html__('POS Cashier', 'posify-for-woocommerce'),
                    array_combine($posCashierPermissions, array_fill(0, count($posCashierPermissions), true))
                );

                // 3. Register pos manager role and permissions
                $posManangerPermissions = array_merge(
                    array_keys(get_role('shop_manager')->capabilities),
                    $this->getPosManagerPermissions()
                );

                add_role(
                    EmployeeRoles::MANAGER,
                   esc_html__('POS Manager', 'posify-for-woocommerce'),
                    array_combine($posManangerPermissions, array_fill(0, count($posManangerPermissions), true))
                );

                // 4. Update version
                update_option($this->roleAndPermissionVersionOption, $this->roleAndPermissionVersion);
            }

            $admin = get_role('administrator');

            foreach ($this->getPosifyPermissions() as $permission) {
                $admin->add_cap($permission);
            }
        });

        add_action('woocommerce_rest_check_permissions', [$this, 'checkPermission'], 10, 4);
        add_action('posify_check_permissions', [$this, 'checkPermission'], 10, 4);
        add_action('user_has_cap', [$this, 'userHasCap'], 10, 4);
    }

    public function userHasCap($allcaps, $caps, $args)
    {
        /**
         * If current version is free version
         * We do not check permission for posify_only_their_data
         */
        if (!posify_is_pro_version()) {
            $allcaps['posify_only_their_data'] = false;
        }

        return $allcaps;
    }

    public function checkPermission(?bool $permission, string $context, $object_id, string $object)
    {
        if ($permission) {
            return $permission;
        }

        $permissionMaps = [
            'product' => [
                'posify_manage_pos' => ['read', 'create', 'edit', 'delete'],
                'posify_manage_store' => ['read', 'create', 'edit', 'delete'],
                'posify_use_pos' => ['read'],
            ],

            'product_cat' => [
                'posify_manage_pos' => ['read', 'create', 'edit', 'delete'],
                'posify_manage_store' => ['read', 'create', 'edit', 'delete'],
                'posify_use_pos' => ['read'],
            ],

            'product_variation' => [
                'posify_manage_pos' => ['read', 'create', 'edit', 'delete'],
                'posify_manage_store' => ['read', 'create', 'edit', 'delete'],
                'posify_use_pos' => ['read'],
            ],

            'attributes' => [
                'posify_manage_pos' => ['read', 'create', 'edit', 'delete'],
                'posify_manage_store' => ['read', 'create', 'edit', 'delete'],
                'posify_use_pos' => ['read'],
            ],

            'shop_order' => [
                'posify_manage_store' => ['read', 'create'],
                'posify_use_pos' => ['read', 'create'],
            ],

            'payment_gateways' => [
                'posify_manage_pos' => ['read', 'create', 'edit', 'delete'],
                'posify_manage_store' => ['read', 'create', 'edit', 'delete'],
                'posify_use_pos' => ['read'],
            ],

            'shipping_methods' => [
                'posify_manage_pos' => ['read', 'create', 'edit', 'delete'],
                'posify_manage_store' => ['read'],
                'posify_use_pos' => ['read'],
            ],

            'settings' => [
                'posify_manage_pos' => ['read', 'create', 'edit', 'delete'],
                'posify_manage_store' => ['read', 'create', 'edit', 'delete'],
                'posify_use_pos' => ['read'],
            ],

            'posify_store' => [
                'posify_manage_pos' => ['read', 'create', 'edit', 'delete'],
                'posify_manage_store' => ['read', 'create', 'edit', 'delete'],
                'posify_use_pos' => ['read'],
            ],

            'posify_receipt' => [
                'posify_manage_pos' => ['read', 'create', 'edit', 'delete'],
                'posify_manage_store' => ['read'],
                'posify_use_pos' => ['read'],
            ],

            'posify_register' => [
                'posify_manage_pos' => ['read', 'create', 'edit', 'delete'],
                'posify_manage_store' => ['read', 'create', 'edit', 'delete'],
                'posify_use_pos' => ['read', 'edit'],
            ],

            'posify_setting' => [
                'posify_manage_pos' => ['read', 'edit'],
                'posify_manage_store' => ['read'],
                'posify_use_pos' => ['read'],
            ],

            'posify_register_session' => [
                'posify_manage_pos' => ['read', 'create', 'edit', 'delete'],
                'posify_manage_store' => ['read', 'create', 'edit', 'delete'],
                'posify_use_pos' => ['read', 'create', 'edit', 'delete'],
            ],

            'posify_register_cash' => [
                'posify_manage_pos' => ['read', 'create'],
                'posify_manage_store' => ['read', 'create'],
                'posify_use_pos' => ['read', 'create'],
            ],

            'posify_employee_store' => [
                'posify_manage_pos' => ['read', 'create', 'edit', 'delete'],
                'posify_manage_store' => ['read', 'create', 'edit', 'delete'],
                'posify_use_pos' => ['read'],
            ],

            'posify_report' => [
                'posify_manage_pos' => ['read'],
                'posify_manage_store' => ['read'],
                'posify_use_pos' => ['read'],
            ],

            'posify_employee' => [
                'posify_manage_pos' => ['read'],
                'posify_manage_store' => ['read'],
                'posify_use_pos' => ['read'],
            ],

            'shop_coupon' => [
                'posify_manage_pos' => ['read', 'create', 'edit', 'delete'],
                'posify_manage_store' => ['read', 'create', 'edit'],
                'posify_use_pos' => ['read', 'create', 'edit'],
            ],

            'user' => [
                'posify_manage_pos' => ['read', 'create', 'edit', 'delete'],
                'posify_manage_store' => ['read', 'create', 'edit'],
                'posify_use_pos' => ['read', 'create', 'edit'],
            ],
        ];

        $caps = array_key_exists($object, $permissionMaps) ? $permissionMaps[$object] : [];
        $permission = false;

        foreach ($caps as $_cap => $_contexts) {
            if (current_user_can($_cap) && in_array($context, (array) $_contexts)) {
                $permission = true;
                break;
            }
        }

        return $permission;
    }
}
