<?php

namespace Posify\Woocommerce\ServiceProviders;

use Posify\Woocommerce\DB;
use Posify\Woocommerce\Utilities\HasInstance;

class PosServiceProvider extends BaseServiceProvider
{
    use HasInstance;

    const PAGE_ID_OPTION_NAME = 'posify_page_id';

    protected $template = 'posify-page.php';

    public function register()
    {
        $this->registerPosScripts();
        $this->enqueuePosScripts();
        $this->dequeuePosScripts();
        $this->loadScriptTagModule();
        $this->createPosPageIfNotExists();
        $this->registerPosPageTemplate();
        $this->addPosOnAdminBar();
    }

    protected function registerPosScripts()
    {
        add_action('wp_enqueue_scripts', function () {
            $hash = posify_get_hash();
            $distBaseUrl = posify_plugin_dist_url();

            // phpcs:ignore WordPress.WP.EnqueuedResourceParameters.MissingVersion
            wp_register_style('posify_pos_main_style', $distBaseUrl . "/pos/app/style.$hash.css", null, null);
            wp_register_script('posify_pos_service_worker_script', $distBaseUrl . '/pos/registerSW.js', null, POSIFY_VERSION, ['in_footer' => true]);

            // phpcs:ignore WordPress.WP.EnqueuedResourceParameters.MissingVersion
            wp_register_script('posify_pos_main_script', $distBaseUrl . "/pos/app/index.$hash.js", null, null, ['in_footer' => true]);
        });
    }

    protected function enqueuePosScripts()
    {
        add_action('wp_enqueue_scripts', function () {
            global $post;

            if (isset($post->ID) && $post->ID != get_option(static::PAGE_ID_OPTION_NAME)) {
                return;
            }

            wp_enqueue_script('posify_pos_main_script');
            wp_enqueue_script('posify_pos_service_worker_script');
            wp_enqueue_style('posify_pos_main_style');

            wp_localize_script('posify_pos_main_script', 'posifyConfig', [
                'env' => 'production',
                'nonce' => posify_create_nonce(),
                'apiURL' => get_rest_url(),
                'adminURL' => get_admin_url() . 'admin.php?page=posify-woocommerce',
                'mode' => posify_mode(),
            ]);
        }, 99);
    }

    protected function dequeuePosScripts()
    {
        add_action('wp_print_scripts', function () {
            global $wp_scripts;
            global $post;

            if (isset($post->ID) && $post->ID == get_option(static::PAGE_ID_OPTION_NAME)) {
                $allowedScripts = apply_filters('posify_pos_allowed_scripts', [
                    'posify_pos_main_script',
                    'posify_pos_service_worker_script',
                ]);

                foreach ($wp_scripts->queue as $script) {
                    if (!in_array($script, $allowedScripts)) {
                        wp_dequeue_script($script);
                    }
                }
            }

        }, 99);

        add_action('wp_print_styles', function () {
            global $wp_styles;
            global $post;

            if (isset($post->ID) && $post->ID == get_option(static::PAGE_ID_OPTION_NAME)) {
                $allowedStyles = apply_filters('posify_pos_allowed_styles', [
                    'posify_pos_main_style',
                ]);

                foreach ($wp_styles->queue as $style) {
                    if (!in_array($style, $allowedStyles)) {
                        wp_dequeue_style($style);
                    }
                }
            }

        }, 99);
    }

    protected function loadScriptTagModule()
    {
        add_filter('script_loader_tag', function ($tag, $handle, $src) {
            if (in_array($handle, [
                'posify_pos_main_script',
            ])) {
                $tag = '<script type="module" crossorigin src="' . esc_url($src) . '"></script>';
            }

            return $tag;
        }, 10, 3);
    }

    protected function createPosPageIfNotExists()
    {
        add_action('init', function () {
            $optionValue = get_option(static::PAGE_ID_OPTION_NAME);

            if ($optionValue && get_post($optionValue)) {
                update_post_meta($optionValue, '_wp_page_template', $this->template);
                return;
            }

            $slug = _x('posify', 'slug of the page', 'posify-for-woocommerce');

            $pageFound = DB::table('posts')
                ->select('ID')
                ->where('post_name', $slug)
                ->where('post_type', 'page')
                ->where('post_status', 'publish')
                ->first();

            if ($pageFound) {
                !$optionValue && update_option(static::PAGE_ID_OPTION_NAME, $pageFound->ID);
            } else {
                $pageData = [
                    'post_status'    => 'publish',
                    'post_type'      => 'page',
                    'post_author'    => 1,
                    'post_name'      => $slug,
                    'post_title'     => esc_html__('Posify POS', 'posify-for-woocommerce'),
                    'post_content'   => '',
                    'post_parent'    => 0,
                    'comment_status' => 'closed'
                ];

                $pageId = wp_insert_post($pageData);
                add_post_meta($pageId, '_wp_page_template', $this->template);
                update_option(static::PAGE_ID_OPTION_NAME, $pageId);
            }
        });

        add_filter('display_post_states', function ($post_states, $post) {
            if ($post->ID == get_option(static::PAGE_ID_OPTION_NAME)) {
                $post_states['custom_sticky'] = esc_html__('Posify POS Page', 'posify-for-woocommerce');
            }

            return $post_states;
        }, 10, 2);
    }

    protected function addPosOnAdminBar()
    {
        add_action('admin_bar_menu', function ($wp_admin_bar) {
            $wp_admin_bar->add_menu(array(
                'id' => 'posify-pos-page',
                'title' => esc_html__('Visit POS', 'posify-for-woocommerce'),
                'href' => get_permalink(get_option(static::PAGE_ID_OPTION_NAME)),
                'parent' => 'site-name',
            ));
        });
    }

    public function registerPosPageTemplate()
    {
        add_filter('theme_page_templates', function ($templates) {
            $templates[$this->template] = 'POS';
            return $templates;
        });

        add_filter('page_template', function ($template) {
            $post = get_post();
            $pageTemplate = get_post_meta($post->ID, '_wp_page_template', true);

            if ($this->template == basename($pageTemplate)) {
                $template = POSIFY_HOME_DIR . '/resources/views/pos.php';
            }

            return $template;
        });
    }
}
