<?php

namespace Posify\Woocommerce\ServiceProviders;

use Posify\Core\Loader;
use Posify\Core\Repositories\RegisterRepository;
use Posify\Woocommerce\Authorization\Auth;
use Posify\Woocommerce\ServiceProviders\BaseServiceProvider;
use Posify\Woocommerce\UseCases\ForceDeleteOrder;
use Posify\Woocommerce\UseCases\RestoreOrder;
use Posify\Woocommerce\UseCases\SoftDeleteOrder;
use Posify\Woocommerce\UseCases\SyncOrder;
use Posify\Woocommerce\Utilities\HasInstance;
use WP_REST_Request;

class RestApiServiceProvider extends BaseServiceProvider
{
    use HasInstance;

    public function register()
    {
        $this->registerRoutes();

        /**
         * Sync order
         *
         * We need to sync order when:
         * - Has new order
         * - Update order
         * - Soft delete order
         * - Force delete order
         * - Restore order
         */
        add_action('woocommerce_new_order', [$this, 'syncOrderWhenHasNewOrder'], 10, 2);
        add_action('woocommerce_update_order', [$this, 'syncOrderWhenUpdateOrder'], 10, 2);
        add_action('trashed_post', [$this, 'syncOrderWhenSoftDeleteOrder'], 10, 2);
        add_action('untrashed_post', [$this, 'syncOrderWhenRestoreOrder'], 10, 1);
        add_action('before_delete_post', [$this, 'syncOrderWhenForceDeleteOrder'], 10, 1);

        add_action('rest_pre_serve_request', [$this, 'allowOriginRequest'], 15);
        add_filter('woocommerce_rest_orders_prepare_object_query', [$this, 'extendOrderQuery'], 10, 2);

        add_filter('woocommerce_rest_prepare_product_object', [$this, 'extendProductRest'], 10, 3);
        add_filter('woocommerce_rest_prepare_product_variation_object', [$this, 'extendProductRest'], 10, 3);
        add_filter('rest_exposed_cors_headers', [$this, 'addPosifyNonceHeader']);

        add_filter('rest_product_collection_params', [$this, 'extendProductPerPage'], 10, 1);
    }

    protected function registerRoutes()
    {
        include_once POSIFY_HOME_DIR . '/routes/api.php';
    }

    public function extendProductPerPage($params)
    {
        $params['per_page']['maximum'] = 9999999 + 1;
        return $params;
    }

    public function allowOriginRequest()
    {
        header('X-Posify-Nonce: ' . posify_create_nonce());

        if (defined('POSIFY_ENV') && POSIFY_ENV == 'development') {
            header('Access-Control-Allow-Origin: *');
        }
    }

    public function addPosifyNonceHeader($headers)
    {
        $headers[] = 'X-Posify-Nonce';
        return $headers;
    }

    public function syncOrderWhenHasNewOrder($orderId, $wcOrder)
    {
        if (!posify_is_pos_order($wcOrder)) {
            return;
        }

        $registerRepository = Loader::load(RegisterRepository::class);
        $registerId = (int) $wcOrder->get_meta('_posify_register_id');
        $cashierId = Auth::check() ? Auth::user()->getId() : null;
        $register = $registerRepository->find($registerId);
        $store = $register->getStore();

        $wcOrder->update_meta_data('_posify_store_id', $store->getId());
        $wcOrder->update_meta_data('_posify_cashier_id', $cashierId);
        $wcOrder->update_meta_data('_posify_store_name', $store->getName());
        $wcOrder->update_meta_data('_posify_register_name', $register->getName());
        $wcOrder->update_meta_data('_posify_cashier_name', Auth::check() ? Auth::user()->getFullname() : null);
        $wcOrder->save();

        SyncOrder::make($wcOrder)->execute();
    }

    public function syncOrderWhenUpdateOrder($orderId, $wcOrder)
    {
        if (!posify_is_pos_order($wcOrder)) {
            return;
        }

        SyncOrder::make($wcOrder)->execute();
    }

    public function syncOrderWhenSoftDeleteOrder($postId)
    {
        if (!posify_is_shop_order($postId)) {
            return;
        }

        SoftDeleteOrder::make($postId)->execute();
    }

    public function syncOrderWhenForceDeleteOrder($postId)
    {
        if (!posify_is_shop_order($postId)) {
            return;
        }

        ForceDeleteOrder::make($postId)->execute();
    }

    public function syncOrderWhenRestoreOrder($postId)
    {
        if (!posify_is_shop_order($postId)) {
            return;
        }

        RestoreOrder::make($postId)->execute();
    }

    public function extendOrderQuery(array $args, WP_REST_Request $request)
    {
        $registerId = $request->get_param('register');
        $fromPosifyChannel = $request->get_param('from_posify_channel');

        if ($registerId) {
            $args['meta_query'][] = array(
                'key'   => '_posify_register_id',
                'value' => $registerId,
            );
        }

        if ($fromPosifyChannel) {
            $args['meta_query'][] = array(
                'key'   => '_posify_source',
                'value' => 'posify',
            );
        }

        return $args;
    }

    public function extendProductRest($response, $product, $request)
    {
        if (static::clientNeedField('posify_purchasable', $request)) {
            $posifyPurchasable = $product->is_purchasable();

            $stockStatus = $product->get_stock_status();

            if ($stockStatus == 'outofstock') {
                $posifyPurchasable = false;
            }

            $posifyPurchasable = apply_filters('posify_product_purchasable', $posifyPurchasable, $product, $request);

            /**
             * posify_purchasable is a key to check if product is purchasable or not
             *
             * Notice: posify_purchasable is very different from with purchasable key of WooCommerce
             * purchasable key of WooCommerce is a key to check if product is purchasable or not even if product is out of stock
             * But posify_purchasable is a key to check if product is purchasable or not base on stock status
             */
            $response->data['posify_purchasable'] = $posifyPurchasable;
        }

        if (static::clientNeedField('posify_variations', $request)) {
            $posifyVariations = [];

            if ($product->is_type('variable')) {
                $productAttributes = $product->get_attributes(true);
                $variations = $product->get_available_variations(true);

                foreach ($variations as $variation) {
                    $titles = [];
                    $posifyVariationOptions = [];

                    foreach ($variation->get_attributes() as $attributeName => $attributeOption) {
                        $titles[] = $attributeOption;
                        $attribute = isset($productAttributes[$attributeName]) ? $productAttributes[$attributeName] : null;

                        if ($attribute) {
                            $posifyVariationOptions[] = [
                                'id' => $attribute->get_id(),
                                'name' => $attribute->get_name(),
                                'option' => $attributeOption
                            ];
                        }
                    }

                    $variationPosifyThumbnail = null;
                    $imageId = $variation->get_image_id();

                    if ($imageId) {
                        $variationPosifyThumbnail = wp_get_attachment_image_url($imageId, 'full');
                    }

                    $variationPosifyPurchasable = $variation->is_purchasable();

                    if ($variation->get_stock_status() == 'outofstock') {
                        $variationPosifyPurchasable = false;
                    }

                    $posifyVariations[] = [
                        'id' => $variation->get_id(),
                        'posify_title' => implode(' / ', $titles),
                        'posify_options' => $posifyVariationOptions,
                        'price' => $variation->get_price(),
                        'sku' => $variation->get_sku(),
                        'regular_price' => $variation->get_regular_price(),
                        'sale_price' => $variation->get_sale_price(),
                        'manage_stock' => $variation->get_manage_stock(),
                        'tax_class' => $variation->get_tax_class(),
                        'tax_status' => $variation->get_tax_status(),
                        'stock_quantity' => $variation->get_stock_quantity(),
                        'stock_status' => $variation->get_stock_status(),
                        'purchasable' => $variation->is_purchasable(),
                        'posify_purchasable' => $variationPosifyPurchasable,
                        'posify_thumbnail' => $variationPosifyThumbnail,
                    ];
                }
            }

            $response->data['posify_variations'] = $posifyVariations;
        }

        if (static::clientNeedField('posify_thumbnail', $request)) {
            $posifyThumbnail = null;
            $imageId = $product->get_image_id();

            if ($imageId) {
                $posifyThumbnail = wp_get_attachment_image_url($imageId, 'full');
            }

            $response->data['posify_thumbnail'] = $posifyThumbnail;
        }

        return $response;
    }

    private static function clientNeedField($field, $request)
    {
        $fields = $request->get_param('_fields');

        /**
         * If client don't send _fields param, we need all fields
         * Or client specify _fields param
         */
        if (count($fields) == 0 || in_array($field, $fields)) {
            return true;
        }

        return false;
    }
}
