<?php

namespace Posify\Core\Utilities;

use Posify\Core\ValueObjects\Undefined;
use ReflectionClass;
use ReflectionException;
use ReflectionIntersectionType;
use ReflectionNamedType;
use ReflectionUnionType;

trait HasMakeFactory
{
    public static function make(array $data = []): self
    {
        $params = static::getConstructorParams($data);
        $self = new static(...array_values($params));

        return $self;
    }

    private static function getConstructorParams(array $initData = []): array
    {
        $defaultValues = [];

        $reflection = new ReflectionClass(static::class);
        $parameters = $reflection->getConstructor()->getParameters();

        foreach ($parameters as $parameter) {
            $typeHint = $parameter->getType();

            try {
                $defaultValue = $parameter->getDefaultValue();
            } catch (ReflectionException $e) {
                $defaultValue = Undefined::make();
            }

            if (self::hasUndefinedTypeHint($typeHint)) {
                $defaultValue = Undefined::make();
            }

            if (isset($initData[$parameter->getName()])) {
                $defaultValue = $initData[$parameter->getName()];
            }

            $defaultValues[$parameter->getName()] = $defaultValue;
        }

        return $defaultValues;
    }

    /**
     * Check if the type hint is Undefined
     *
     * @param ReflectionNamedType|ReflectionUnionType|ReflectionIntersectionType $type
     * @return bool
     */
    private static function hasUndefinedTypeHint($type): bool
    {
        if (! $type) {
            return false;
        }

        if (
            $type instanceof ReflectionUnionType
            || $type instanceof ReflectionIntersectionType
        ) {
            $typesHint = $type->getTypes();

            foreach ($typesHint as $typeHint) {
                if (str_contains($typeHint->getName(), Undefined::class)) {
                    return true;
                }
            }

            return false;
        }

        return str_contains($type->getName(), Undefined::class);
    }
}
