<?php

if (!defined('ABSPATH')) exit;

use Automattic\WooCommerce\Internal\DataStores\Orders\CustomOrdersTableController;
use Automattic\WooCommerce\Utilities\OrderUtil;
use Carbon\Carbon;
use Carbon\Exceptions\InvalidFormatException;
use Posify\Core\DataTransferObjects\AddRegisterCashData;
use Posify\Core\DataTransferObjects\AddStoreEmployeeData;
use Posify\Core\DataTransferObjects\CreateEmployeeData;
use Posify\Core\DataTransferObjects\CreateReceiptData;
use Posify\Core\DataTransferObjects\CreateRegisterData;
use Posify\Core\DataTransferObjects\CreateStockData;
use Posify\Core\DataTransferObjects\CreateStoreData;
use Posify\Core\DataTransferObjects\UpdateReceiptData;
use Posify\Core\DataTransferObjects\UpdateRegisterData;
use Posify\Core\DataTransferObjects\UpdateSettingData;
use Posify\Core\DataTransferObjects\UpdateStockData;
use Posify\Core\DataTransferObjects\UpdateStoreData;
use Posify\Core\DataTransferObjects\UpdateStoreEmployeeData;
use Posify\Core\Entities\Employee;
use Posify\Core\Entities\Receipt;
use Posify\Core\Entities\Register;
use Posify\Core\Entities\RegisterSession;
use Posify\Core\Entities\Stock;
use Posify\Core\Entities\Store;
use Posify\Core\Exceptions\DataInvalidException;
use Posify\Core\UseCases\AddRegisterCash;
use Posify\Core\UseCases\AddStoreEmployee;
use Posify\Core\UseCases\CreateEmployee;
use Posify\Core\UseCases\CreateReceipt;
use Posify\Core\UseCases\CreateRegister;
use Posify\Core\UseCases\CreateStock;
use Posify\Core\UseCases\CreateStore;
use Posify\Core\UseCases\DecreaseStockQuantity;
use Posify\Core\UseCases\DeleteReceipt;
use Posify\Core\UseCases\DeleteRegister;
use Posify\Core\UseCases\DeleteStore;
use Posify\Core\UseCases\DeleteStoreEmployee;
use Posify\Core\UseCases\IncreaseStockQuantity;
use Posify\Core\UseCases\UpdateReceipt;
use Posify\Core\UseCases\UpdateRegister;
use Posify\Core\UseCases\UpdateSetting;
use Posify\Core\UseCases\UpdateStock;
use Posify\Core\UseCases\UpdateStore;
use Posify\Core\UseCases\UpdateStoreEmployee;
use Posify\Woocommerce\ServiceProviders\AffiliateServiceProvider;
use Posify\Woocommerce\ServiceProviders\PosServiceProvider;
use Posify\Woocommerce\ServiceProviders\PosifyServiceProvider;

/**
 * -----------------------------------------------
 * Posify Utilities
 * -----------------------------------------------
 */

if (!function_exists('posify_mode')) {
    function posify_mode()
    {
        return apply_filters('posify_mode', 'live');
    }
}

if (!function_exists('posify_view')) {
    function posify_view($path, $data = [])
    {
        extract($data);
        include POSIFY_HOME_DIR . '/resources/views/' . $path . '.php';
    }
}

if (!function_exists('posify_get_view')) {
    function posify_get_view($path, $data = [])
    {
        ob_start();

        extract($data);
        include POSIFY_HOME_DIR . '/resources/views/' . $path . '.php';

        return ob_get_clean();
    }
}

if (!function_exists('posify_plugin_dist_url')) {
    function posify_plugin_dist_url($append = '')
    {
        return plugin_dir_url(__FILE__) . 'dist' . $append;
    }
}

if (!function_exists('posify_plugin_dist_path')) {
    function posify_plugin_dist_path($append = '')
    {
        return POSIFY_HOME_DIR . '/dist' . $append;
    }
}


if (!function_exists('posify_get_hash')) {
    function posify_get_hash()
    {
        $hashFilePath = posify_plugin_dist_path('/pos/hash.json');

        if (!file_exists($hashFilePath)) {
            return null;
        }

        // phpcs:ignore WordPress.WP.AlternativeFunctions.file_get_contents_file_get_contents
        $hash = json_decode(file_get_contents($hashFilePath), true);

        if (!isset($hash['hash'])) {
            return null;
        }

        return $hash['hash'];
    }
}

if (!function_exists('posify_controller_action')) {
    function posify_controller_action($controller, $action)
    {
        return [new $controller, $action];
    }
}

if (!function_exists('posify_route')) {
    function posify_route($method, $path, $callback, $permission = 'is_user_logged_in')
    {
        add_action('rest_api_init', function () use ($method, $path, $callback, $permission) {
            register_rest_route('posify/v1', $path, [
                'methods' => $method,
                'callback' => $callback,
                'permission_callback' => $permission,
            ]);
        });
    }
}

if (!function_exists('posify_wc_get_payment_gateway_title_by_id')) {
    function posify_wc_get_payment_gateway_title_by_id($gateway_id)
    {
        $payment_gateways = WC_Payment_Gateways::instance();

        if (!isset($payment_gateways->payment_gateways)) {
            return null;
        }

        foreach ($payment_gateways->payment_gateways as $gateway) {
            if ($gateway->id == $gateway_id) {
                return $gateway->get_method_title();
            }
        }

        return null;
    }
}

if (!function_exists('posify_is_pos_order')) {
    function posify_is_pos_order($order)
    {
        return $order->get_meta('_posify_source') == 'posify';
    }
}

if (!function_exists('posify_date_format')) {
    function posify_date_format($format, $date)
    {
        try {
            return Carbon::createFromFormat($format, $date);
        } catch (InvalidFormatException $e) {
            //
        }

        return null;
    }
}

if (!function_exists('posify_get_previous_date')) {
    function posify_get_previous_date(Carbon $startDate, Carbon $endDate)
    {
        $diff = $startDate->diffInDays($endDate) + 1;

        $previousStartDate = $startDate->copy()->subDays($diff);
        $previousEndDate = $endDate->copy()->subDays($diff);

        return [
            $previousStartDate,
            $previousEndDate,
        ];
    }
}

if (!function_exists('posify_create_nonce')) {
    function posify_create_nonce()
    {
        return wp_create_nonce('wp_rest');
    }
}

if (!function_exists('posify_pos_head_tags')) {
    function posify_pos_head_tags()
    {
        return implode("\n", PosServiceProvider::instance()->headTags());
    }
}

if (!function_exists('posify_pos_before_body_tags')) {
    function posify_pos_before_body_tags()
    {
        return implode("\n", PosServiceProvider::instance()->beforeBodyTags());
    }
}

if (!function_exists('posify_pos_after_body_tags')) {
    function posify_pos_after_body_tags()
    {
        return implode("\n", PosServiceProvider::instance()->afterBodyTags());
    }
}

if (!function_exists('posify_pos_body_class')) {
    function posify_pos_body_class()
    {
        return implode(' ', PosServiceProvider::instance()->bodyClass());
    }
}

if (!function_exists('posify_is_shop_order')) {
    function posify_is_shop_order($postId)
    {
        if (class_exists(OrderUtil::class) && OrderUtil::custom_orders_table_usage_is_enabled()) {
            return ('shop_order' === OrderUtil::get_order_type($postId));
        }

        return ('shop_order' === get_post_type($postId));
    }
}

if (!function_exists('posify_get_shop_order_screen')) {
    function posify_get_shop_order_screen()
    {
        if (
            class_exists(CustomOrdersTableController::class)
            && wc_get_container()->get(CustomOrdersTableController::class)->custom_orders_table_usage_is_enabled()
        ) {
            return wc_get_page_screen_id('shop-order');
        }

        return 'shop_order';
    }
}

if (!function_exists('posify_now')) {
    function posify_now()
    {
        return Carbon::createFromFormat('Y-m-d H:i:s', current_time('mysql'));
    }
}

if (!function_exists('posify_get_plan')) {
    function posify_get_plan()
    {
        $plan = 'free';
        $proScriptPath = posify_plugin_dist_path('/admin/pro-version/app/main.js');

        if (file_exists($proScriptPath)) {
            $plan = 'pro';
        }

        return apply_filters('posify_plan', $plan);
    }
}

if (!function_exists('posify_is_pro_version')) {
    function posify_is_pro_version()
    {
        return posify_get_plan() === 'pro';
    }
}

/**
 * -----------------------------------------------
 * Core use case functions
 * -----------------------------------------------
 */

if (!function_exists('POSIFY_INIT')) {
    function POSIFY_INIT()
    {
        return PosifyServiceProvider::instance()->register();
    }
}

if (!function_exists('posify_create_store')) {
    function posify_create_store($data)
    {
        return CreateStore::make(CreateStoreData::make($data))->handle();
    }
}

if (!function_exists('posify_update_store')) {
    function posify_update_store(Store $store, $data)
    {
        return UpdateStore::make($store, UpdateStoreData::make($data))->handle();
    }
}

if (!function_exists('posify_delete_store')) {
    function posify_delete_store(Store $store)
    {
        return DeleteStore::make($store)->handle();
    }
}

if (!function_exists('posify_create_employee')) {
    function posify_create_employee($data)
    {
        try {
            $result = CreateEmployee::make(CreateEmployeeData::make($data))->handle();
        } catch (DataInvalidException $e) {
            $result = new WP_Error(
                $e->getCode(),
                $e->getMessage(),
                [
                    'status' => 422,
                    'details' => $e->getDetailErrors(),
                ]
            );
        }

        return $result;
    }
}

if (!function_exists('posify_add_employee_to_store')) {
    function posify_add_employee_to_store(Store $store, Employee $employee, $enabled = true)
    {
        $storeEmployeeData = AddStoreEmployeeData::make([
            'enabled' => $enabled,
        ]);

        return AddStoreEmployee::make($store, $employee, $storeEmployeeData)->handle();
    }
}

if (!function_exists('posify_update_employee_in_store')) {
    function posify_update_employee_in_store(Store $store, Employee $employee, $enabled = true)
    {
        $storeEmployeeData = UpdateStoreEmployeeData::make([
            'enabled' => $enabled,
        ]);

        return UpdateStoreEmployee::make($store, $employee, $storeEmployeeData)->handle();
    }
}

if (!function_exists('posify_remove_employee_from_store')) {
    function posify_remove_employee_from_store(Store $store, Employee $employee)
    {
        return DeleteStoreEmployee::make($store, $employee)->handle();
    }
}

if (!function_exists('posify_create_receipt')) {
    function posify_create_receipt($data)
    {
        return CreateReceipt::make(CreateReceiptData::make($data))->handle();
    }
}

if (!function_exists('posify_update_receipt')) {
    function posify_update_receipt(Receipt $receipt, $data)
    {
        return UpdateReceipt::make($receipt, UpdateReceiptData::make($data))->handle();
    }
}

if (!function_exists('posify_delete_receipt')) {
    function posify_delete_receipt(Receipt $receipt)
    {
        try {
            return DeleteReceipt::make($receipt)->handle();
        } catch (DataInvalidException $e) {
            return new WP_Error($e->getCode(), $e->getMessage());
        }
    }
}

if (!function_exists('posify_add_cash_to_register_session')) {
    function posify_add_cash_to_register_session(RegisterSession $currentSession, $data)
    {
        return AddRegisterCash::make(
            $currentSession,
            AddRegisterCashData::make($data)
        )->handle();
    }
}


if (!function_exists('posify_create_register')) {
    function posify_create_register($data)
    {
        return CreateRegister::make(CreateRegisterData::make($data))->handle();
    }
}

if (!function_exists('posify_update_register')) {
    function posify_update_register(Register $register, $data)
    {
        return UpdateRegister::make($register, UpdateRegisterData::make($data))->handle();
    }
}

if (!function_exists('posify_delete_register')) {
    function posify_delete_register(Register $register)
    {
        return DeleteRegister::make($register)->handle();
    }
}

if (!function_exists('posify_update_setting')) {
    function posify_update_setting($data)
    {
        UpdateSetting::make(UpdateSettingData::make($data))->handle();
    }
}

if (!function_exists('posify_create_stock')) {
    function posify_create_stock($data)
    {
        CreateStock::make(CreateStockData::make($data))->handle();
    }
}

if (!function_exists('posify_update_stock')) {
    function posify_update_stock(Stock $stock, $data)
    {
        UpdateStock::make($stock, UpdateStockData::make($data))->handle();
    }
}

if (!function_exists('posify_increase_stock_quantity')) {
    function posify_increase_stock_quantity(Stock $stock, $itemStockReduced)
    {
        return IncreaseStockQuantity::make($stock, $itemStockReduced)->handle();
    }
}

if (!function_exists('posify_decrease_stock_quantity')) {
    function posify_decrease_stock_quantity(Stock $stock, $itemStockReduced)
    {
        return DecreaseStockQuantity::make($stock, $itemStockReduced)->handle();
    }
}

if (!function_exists('posify_get_plugin_entire_file')) {
    function posify_get_plugin_entire_file()
    {
        return POSIFY_SLUG . '/index.php';
    }
}

if (!function_exists('posify_get_affiliate_code')) {
    function posify_get_affiliate_code()
    {
        return get_option(AffiliateServiceProvider::$affiliateCodeOptionKey, null);
    }
}
